/*
The 8x8x8 RGB LED Cube

by Kevin Darrah
www.kevindarrah.com
kevin@kevindarrah.com

Layer select conversion for SuperTech-IT Printed Circuit board
by Norman Skinner - August 2014
Also porting over to Arduino MEGA2560
www.TheLEDCube.com
SuperTech@TheLEDCube.com

MSGEQ7 control code by David Yee
Music response animations by SuperTech-IT ported over from ChipKit UNO32 by David Yee - July 2015.

Latest
*** Ver 12 RevA 8/8/2014

Release Notes:
V11
- Fixed bug with BAM timing
- Moved Blank pin setup to ISR so pins are dead until they are written to

V12
bitwise operation to set pins LOW was incorrect
should be PORTx &= ~(1<<pinNumber);


Pin definition clarification (SuperTech-IT)
Layer 8 - Uno pin - A5 is the same ad D19 - but in an array, we must use 19 to specify A5.

Disclaimer:
Not a professional LED cube builder, this is how I did it, and this is my cube
*/



#include <SPI.h>// SPI Library used to clock data out to the shift registers

#define latch_pin 4// Defines actual BIT of PortD for latch - is Arduino UNO pin 2, MEGA pin 4
#define blank_pin 5// Defines actual BIT of PortD for blank - is Arduino UNO pin 3, MEGA pin 5
#define data_pin 51// used by SPI, must be pin MOSI 11 on Arduino UNO, 51 on MEGA
#define clock_pin 52// used by SPI, must be 13 SCK 13 on Arduino UNO, 52 on MEGA

/* *** these assign layers to their digital pin numbers - all other programming should use these definitions
Added by SuperTech-IT for parallel layer data output to conform to RGB LED Cube PCB */
#define layer1 4 // bottom layer
#define layer2 5
#define layer3 6
#define layer4 7
#define layer5 8
#define layer6 9
#define layer7 10
#define layer8 11 // top layer
int layerArray[8] = {layer1,layer2,layer3,layer4,layer5,layer6,layer7,layer8};
int lastAnode;
// uint16_t layerbyname[8]={PG5,   PE3,   PH3,   PH4,   PH5,   PH6,   PB4, PB5};
// volatile uint8_t * const portbyname[8] = {&PORTG, &PORTE, &PORTH, &PORTH, &PORTH, &PORTH, &PORTB, &PORTB};


// End of Layer Definitions for SuperTech-IT board

//***variables***variables***variables***variables***variables***variables***variables***variables
//These variables are used by multiplexing and Bit Angle Modulation Code
int shift_out;//used in the code a lot in for(i= type loops
byte anode[8];//byte to write to the anode shift register, 8 of them, shifting the ON level in each byte in the array

//This is how the brightness for every LED is stored,  
//Each LED only needs a 'bit' to know if it should be ON or OFF, so 64 Bytes gives you 512 bits= 512 LEDs
//Since we are modulating the LEDs, using 4 bit resolution, each color has 4 arrays containing 64 bits each
byte red0[64], red1[64], red2[64], red3[64];
byte blue0[64], blue1[64], blue2[64], blue3[64];
byte green0[64], green1[64], green2[64], green3[64];
//notice how more resolution will eat up more of your precious RAM

int level=0;//keeps track of which level we are shifting data to
int anodeLevel=0;//this increments through the anode levels
int BAM_Bit, BAM_Counter=0; // Bit Angle Modulation variables to keep track of things

//These variables can be used for other things
unsigned long start;//for a millis timer to cycle through the animations

//These next items support SuperTech-IT's music module


//***********************************************************************************************************************
#define MSGEQ7 0x01  // define if music module exists (1) or not (0)
//***********************************************************************************************************************
// If this is set to 1 and there is NO module attached, the cube will hang after 1 or two iterations of the first effect.
//***********************************************************************************************************************

const int Gain = A5;   // pin where gain pot is
int virtualGain = 127; // define initial gain status until it is read from A5
int VU[7];             // music input level global array for 7 frequencies ****  Make this a local variable  ******
int VUtop[7];          // music level refined to a value of 0 to 8 depending on input level and gain, checked to not exceed 8.
int VUtopb[8];         // music level with an average frequency between band 0 and 1 for routines needing 8 frequencies. 
int runMode = 0;       // runMode 0 = animations. 1=cycle through music modes. 2 or higher indicates which nusic mode to run and stay in.
int modes = 12;        // The number of total music modes there are. Don't forget to add 1 for the sequence mode - 
                       // so if there are 3 modes of music, this has to be 4
int beat = 0;          // global beat detect = 1 during beat, otherwise 0.
int threshold = 900;   // this is the threshold from 0 to 1023 of how high the bass must be to consider it a "beat"

//***************************************************************************************************************************
// Domeke Code Integration modified for Arduino Mega
//***************************************************************************************************************************
// These are standard colors which can be called by name when using the getColor subroutine.
#define Black  191
#define Red 0
#define Orange  21
#define Yellow  40
#define Green 63
#define Aqua  85
#define Blue  127
#define Violet  148
#define Purple  169
#define White 190
int myred, mygreen, myblue; // these are where the getColor routine returns its color components
int mycolor;
byte mycase;

//****setup****setup****setup****setup****setup****setup****setup****setup****setup****setup****setup****setup****setup
void setup(){

SPI.setBitOrder(MSBFIRST);//Most Significant Bit First
SPI.setDataMode(SPI_MODE0);// Mode 0 Rising edge of data, keep clock low
SPI.setClockDivider(SPI_CLOCK_DIV2);//Run the data in at 16MHz/2 - 8MHz

noInterrupts();// kill interrupts until everybody is set up

//We use Timer 1 to refresh the cube
TCCR1A = B00000000;//Register A all 0's since we're not toggling any pins
TCCR1B = B00001011;//bit 3 set to place in CTC mode, will call an interrupt on a counter match
//bits 0 and 1 are set to divide the clock by 64, so 16MHz/64=250kHz
TIMSK1 = B00000010;//bit 1 set to call the interrupt on an OCR1A match
OCR1A=43; // you can play with this, but I set it to 30, which means:
//our clock runs at 250kHz, which is 1/250kHz = 4us
//with OCR1A set to 30, this means the interrupt will be called every (30+1)x4us=124us, 
// which gives a multiplex frequency of about 8kHz
// originally =45.  Making it larger may affect the music routines.

// here I just set up the anode array, this is what's written to the anode shift register, to enable each level
anode[0]=B11111110;
anode[7]=B11111101;
anode[6]=B11111011;
anode[5]=B11110111;
anode[4]=B11101111;
anode[3]=B11011111;
anode[2]=B10111111;
anode[1]=B01111111;
// don't hate on how I assigned the values to this register! haha

//finally set up the Outputs
// pinMode(latch_pin, OUTPUT);//Latch
pinMode (2, OUTPUT); // turn off PWM and set PortD bit 4 as output
pinMode (3, OUTPUT); // turn off PWM and set PortD bit 5 as output
pinMode(data_pin, OUTPUT);//MOSI DATA
pinMode(clock_pin, OUTPUT);//SPI Clock
//pinMode(blank_pin, OUTPUT);//Output Enable  important to do this last, so LEDs do not flash on boot up

//*** Here layer pins are set as outputs
pinMode(layer1, OUTPUT);
pinMode(layer2, OUTPUT);
pinMode(layer3, OUTPUT);
pinMode(layer4, OUTPUT);
pinMode(layer5, OUTPUT);
pinMode(layer6, OUTPUT);
pinMode(layer7, OUTPUT);
pinMode(layer8, OUTPUT);


if (MSGEQ7 == 1) 
   {// leave this stuff alone if the module is not present
    // This stuff sets up pins for SuperTech-ITs music module. 
    pinMode (A0,INPUT); // where we read the analog MSGEQ7's output
    pinMode (A1, OUTPUT); // MSGEQ7 RESET line
    pinMode (A2, INPUT); // music mode button
    pinMode (A3, OUTPUT); // MSGEQ7 strobe line
    pinMode (A4, INPUT); // animation mode button
    pinMode (A5, INPUT); // Virtual Gain potentiometer
    digitalWrite (A1, HIGH); // on startup put the MSGEQ7 in reset mode
    digitalWrite (A3, HIGH); // Ready the active low strobe for first   
   }
   //  end MSGEQ7 prep
   
SPI.begin();//start up the SPI library
interrupts();//let the show begin, this lets the multiplexing start
}//***end setup***end setup***end setup***end setup***end setup***end setup***end setup***end setup***end setup***end setup


void loop(){//***start loop***start loop***start loop***start loop***start loop***start loop***start loop***start loop***start loop
// Each animation located in a sub routine
// To control an LED, you simply:
// LED(level you want 0-7, row you want 0-7, column you want 0-7, red brighness 0-15, green brighness 0-15, blue brighness 0-15);
  while (runMode > 0) {
  byte musicMode = runMode-1; // musicMode set to make it easier to tell which routine number is running.
  fill(0,0,0); // if jumping out of animation, clear cube
  if ((runMode == 1) || (musicMode == 1)) {spectrum1 (1000,0);} // run if in sequential mode or constant music mode 1
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 2)) {spectrum2 (500,0);} // run if in sequential mode or constant music mode 2
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 3)) {spectrum3 (350,0);} // run if in sequential mode or constant music mode 3
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 4)) {spectrum3a (1000,0);} // run if in sequential mode or constant music mode 3
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 5)) {spectrum4 (500,0);} // run if in sequential mode or constant music mode 4
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 6)) {spectrum5 (500,0);} // run if in sequential mode or constant music mode 5
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 7)) {spectrum5a (500,0);} // run if in sequential mode or constant music mode 5
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 8)) {spectrum6 (1000,0);} // run if in sequential mode or constant music mode 6
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 9)) {spectrum7 (500,10,5);} // run if in sequential mode or constant music mode 7
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
 
  if ((runMode == 1) || (musicMode == 10)) {spectrum9 (500,0);} // run if in sequential mode or constant music mode 9
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  if ((runMode == 1) || (musicMode == 11)) {spectrum10 (500,0);} // run if in sequential mode or constant music mode 10
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode

 // spectrum8 is questionable.  Originally from the chipkit music set.  Left here as reference
 // NOTE WHEN ACTIVATING THESE, INCREASE MODES VARIABLE OTHERWISE LOOPING WON'T WORK.
  /*
  if ((runMode == 1) || (musicMode == 12)) {spectrum8 (200,0);} // run if in sequential mode or constant music mode 8
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  */
  
  // Beat mode routine that simply flashes.  Left here as reference.
  /*
  if ((runMode == 1) || (musicMode == 13)) {beat2 (300,2);} // run if in sequential mode or constant music mode 12
  if (runMode == 1) {fill (0,0,0);} // only clear the cube if we are in sequence mode
  */
  } // While runMode < 0
  
  
 while (runMode == 0) 
 {
 fill (0,0,0); // in case exiting music mode, clear the cube
 if (runMode > 0) {break;} // drop out if mode changes
 fireworks (20,15,0);
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 rainVersionTwo();
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 folder(); // aka filop flop
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 sinwaveTwo();
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 wipe_out(); // bouncing balls
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 bouncyvTwo(); // aka snake with colors
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 color_wheelTWO();
 if (runMode > 0) {break;} // drop out if mode changes
 fill (0,0,0);
 harlem_shake();
 if (runMode > 0) {break;} // drop out if mode changes
 // wait 1/2 second before starting over
 delay(500);
} // While Runmode=0

}//***end loop***end loop***end loop***end loop***end loop***end loop***end loop***end loop***end loop***end loop***end loop***end loop



void LED(int row, int column, int level, byte red, byte green, byte blue){ //****LED Routine****LED Routine****LED Routine****LED Routine
//This is where it all starts
//This routine is how LEDs are updated, with the inputs for the LED location and its R G and B brightness levels

// First, check and make sure nothing went beyond the limits, just clamp things at either 0 or 7 for location, and 0 or 15 for brightness
  if(level<0)
  level=0;
  if(level>7)
  level=7;
  if(row<0)
  row=0;
  if(row>7)
  row=7;
  if(column<0)
  column=0;
  if(column>7)
  column=7;  
    if(red<0)
  red=0;
  if(red>15)
  red=15;
  if(green<0)
  green=0;
  if(green>15)
  green=15;
  if(blue<0)
  blue=0;
  if(blue>15)
  blue=15;  
    
  //There are 512 LEDs in the cube, so when we write to level 2, column 5, row 4, that needs to be translated into a number from 0 to 511
  
  //This looks confusing, I know...

// This next variable is the LED number 0-511 = (level*64)+(row*8)+column
// Use shift arithmetic since it's faster than multiply.
   int wholebyte = (level << 6)+(row << 3)+column;  // (level*64)+(row*8)+column
   int whichbyte = int(wholebyte >> 3);             // Here we're dividing by 8
   byte whichbit = wholebyte - (whichbyte << 3);    // which bit = wholebyte - 8*whichbyte

  // The first level LEDs are first in the sequence, then 2nd level, then third, and so on
  //the (level*64) is what indexes the level's starting place, so level 0 are LEDs 0-63, level 1 are LEDs 64-127, and so on
  
  //The column counts left to right 0-7 and the row is back to front 0-7
  //This means that if you had level 0, row 0, the bottom back row would count from 0-7, 
  
  //so if you looked down on the cube, and only looked at the bottom level
  // 00 01 02 03 04 05 06 07
  // 08 09 10 11 12 13 14 15
  // 16 17 18 19 20 21 22 23
  // 24 25 26 27 28 29 30 31
  // 32 33 34 35 36 37 38 39
  // 40 41 42 43 44 45 46 47  
  // 48 49 50 51 52 53 54 55  
  // 56 57 58 59 60 61 62 63

//Then, if you incremented the level, the top right of the grid above would start at 64
//The reason for doing this, is so you don't have to memorize a number for each LED, allowing you to use level, row, column

//Now, what about the divide by 8 in there?
//...well, we have 8 bits per byte, and we have 64 bytes in memory for all 512 bits needed for each LED, so
//we divide the number we just found by 8, and take the integ7er of it, so we know which byte, that bit is located
//confused? that's ok, let's take an example, if we wanted to write to the LED to the last LED in the cube, we would write a 7, 7, 7
// giving (7*64)+(7*8)=7 = 511, which is right, but now let's divide it by 8, 511/8 = 63.875, and take the int of it so, we get 63,
//this is the last byte in the array, which is right since this is the last LED
  
//This will all make sense in a sec
 
 //This is 4 bit color resolution, so each color contains x4 64 byte arrays, explanation below:
  bitWrite(red0[whichbyte], whichbit, bitRead(red, 0));
  bitWrite(red1[whichbyte], whichbit, bitRead(red, 1));
  bitWrite(red2[whichbyte], whichbit, bitRead(red, 2)); 
  bitWrite(red3[whichbyte], whichbit, bitRead(red, 3)); 

  bitWrite(green0[whichbyte], whichbit, bitRead(green, 0));
  bitWrite(green1[whichbyte], whichbit, bitRead(green, 1));
  bitWrite(green2[whichbyte], whichbit, bitRead(green, 2)); 
  bitWrite(green3[whichbyte], whichbit, bitRead(green, 3));

  bitWrite(blue0[whichbyte], whichbit, bitRead(blue, 0));
  bitWrite(blue1[whichbyte], whichbit, bitRead(blue, 1));
  bitWrite(blue2[whichbyte], whichbit, bitRead(blue, 2)); 
  bitWrite(blue3[whichbyte], whichbit, bitRead(blue, 3));
  
//Are you now more confused?  You shouldn't be!  It's starting to make sense now.  Notice how each line is a bitWrite, which is,
//bitWrite(the byte you want to write to, the bit of the byte to write, and the 0 or 1 you want to write)
//This means that the 'whichbyte' is the byte from 0-63 in which the bit corresponding to the LED from 0-511
//Is making sense now why we did that? taking a value from 0-511 and converting it to a value from 0-63, since each LED represents a bit in 
//an array of 64 bytes.
//Then next line is which bit 'wholebyte-(8*whichbyte)'  
//This is simply taking the LED's value of 0-511 and subracting it from the BYTE its bit was located in times 8
//Think about it, byte 63 will contain LEDs from 504 to 511, so if you took 505-(8*63), you get a 1, meaning that,
//LED number 505 is is located in bit 1 of byte 63 in the array

//is that it?  No, you still have to do the bitRead of the brightness 0-15 you are trying to write,
//if you wrote a 15 to RED, all 4 arrays for that LED would have a 1 for that bit, meaning it will be on 100%
//This is why the four arrays read 0-4 of the value entered in for RED, GREEN, and BLUE
//hopefully this all makes some sense?
//********************************************************************************************************************************************
// David Yee's  note: On the Supertech-IT board, if you set Red0[i] or any other color to a non-zero value, it will affect the  bits of byte i where 
// i goes from 0 to 63.   Red0[1] through Red0[63] pertain to the first panel and the whichbit will correspond to a column or row depending 
// on your perspective.  Thus, the first layer on the bottom of the cube will be Red0[0] through Red0[7] and if you want to address the Red0 bit of 
// an LED on that layer, you need to know the value of whichbit.   So, Red0[0] bit0 is a corner LED on the panel.  Red0[1] bit 0 is the next LED
// on the bottom row of the same panel.  Red0[0] bit 1 will be the next led perpendicular to the corner led with respect to the line formed by
// Red0[0] bit 0 and Red0[1] bit 0.  The previous diagram is confusing because it implies that 0-63 correspond to the 
// lowest layer when it really pertains to a panel on the Supertech-IT board.  Then again, the original documentation pertained to a different
// board.
//
// So, the bottom layer on the Supertech-IT board will look like;
//
//  Byte7Bit8  Byte7Bit7  Byte7Bit6  Byte7Bit5  Byte7Bit4  Byte7Bit3  Byte7Bit2  Byte7Bit1  Byte7Bit0  <== Byte7
//  Byte6Bit8  Byte6Bit7  Byte6Bit6  Byte6Bit5  Byte6Bit4  Byte6Bit3  Byte6Bit2  Byte6Bit1  Byte6Bit0  <== Byte6
//  Byte5Bit8  Byte5Bit7  Byte5Bit6  Byte5Bit5  Byte5Bit4  Byte5Bit3  Byte5Bit2  Byte5Bit1  Byte5Bit0  <== Byte5
//  Byte4Bit8  Byte4Bit7  Byte4Bit6  Byte4Bit5  Byte4Bit4  Byte4Bit3  Byte4Bit2  Byte4Bit1  Byte4Bit0  <== Byte4
//  Byte3Bit8  Byte3Bit7  Byte3Bit6  Byte3Bit5  Byte3Bit4  Byte3Bit3  Byte3Bit2  Byte3Bit1  Byte3Bit0  <== Byte3
//  Byte2Bit8  Byte2Bit7  Byte2Bit6  Byte2Bit5  Byte2Bit4  Byte2Bit3  Byte2Bit2  Byte2Bit1  Byte2Bit0  <== Byte2
//  Byte1Bit8  Byte1Bit7  Byte1Bit6  Byte1Bit5  Byte1Bit4  Byte1Bit3  Byte1Bit2  Byte1Bit1  Byte1Bit0  <== Byte1
//  Byte0Bit8  Byte0Bit7  Byte0Bit6  Byte0Bit5  Byte0Bit4  Byte0Bit3  Byte0Bit2  Byte0Bit1  Byte0Bit0  <== Byte0
//
//  Bytes 8-15 correspond to the next layer up and so on.
//*******************************************************************************************************************************************
}//****LED routine end****LED routine end****LED routine end****LED routine end****LED routine end****LED routine end****LED routine end



ISR(TIMER1_COMPA_vect){//***MultiPlex BAM***MultiPlex BAM***MultiPlex BAM***MultiPlex BAM***MultiPlex BAM***MultiPlex BAM***MultiPlex BAM

//This routine is called in the background automatically at frequency set by OCR1A
//In this code, I set OCR1A to 30, so this is called every 124us, giving each level in the cube 124us of ON time
//There are 8 levels, so we have a maximum brightness of 1/8, since the level must turn off before the next level is turned on
//The frequency of the multiplexing is then 124us*8=992us, or 1/992us= about 1kHz


  PORTE |= 1<<blank_pin;//The first thing we do is turn all of the LEDs OFF, by writing a 1 to the blank pin
  //Note, in my bread-boarded version, I was able to move this way down in the cube, meaning that the OFF time was minimized
  //due to signal integrity and parasitic capcitance, my rise/fall times, required all of the LEDs to first turn off, before updating
  //otherwise you get a ghosting effect on the previous level

//This is 4 bit 'Bit angle Modulation' or BAM, There are 8 levels, so when a '1' is written to the color brightness, 
//each level will have a chance to light up for 1 cycle, the BAM bit keeps track of which bit we are modulating out of the 4 bits
//Bam counter is the cycle count, meaning as we light up each level, we increment the BAM_Counter
if(BAM_Counter==8)
BAM_Bit++;
else
if(BAM_Counter==24)
BAM_Bit++;
else
if(BAM_Counter==56)
BAM_Bit++;

BAM_Counter++;//Here is where we increment the BAM counter

switch (BAM_Bit){//The BAM bit will be a value from 0-3, and only shift out the arrays corresponding to that bit, 0-3
//Here's how this works, each case is the bit in the Bit angle modulation from 0-4, 
//Next, it depends on which level we're on, so the byte in the array to be written depends on which level, but since each level contains 64 LED,
//we only shift out 8 bytes for each color
case 0:
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(red0[shift_out]);
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(green0[shift_out]); 
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(blue0[shift_out]);
  break;
  break;
case 1:
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(red1[shift_out]);
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(green1[shift_out]); 
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(blue1[shift_out]);
  break;
 case 2:
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(red2[shift_out]);
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(green2[shift_out]); 
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(blue2[shift_out]);
 break;
 case 3:
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(red3[shift_out]);
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(green3[shift_out]); 
 for(shift_out=level; shift_out<level+8; shift_out++)
 SPI.transfer(blue3[shift_out]);
 //Here is where the BAM_Counter is reset back to 0, it's only 4 bit, but since each cycle takes 8 counts,
 //, it goes 0 8 16 32, and when BAM_counter hits 64 we reset the BAM
  if(BAM_Counter==64){
  BAM_Counter=0;
  BAM_Bit=0;
  }
  break;
}//switch_case

//SPI.transfer(anode[anodeLevel]);//finally, send out the anode level byte

// ** This routine selects layer without shift register. 
    lastAnode = (anodeLevel-1);
    if (anodeLevel == 0) { lastAnode = 7; } // if we are at the bottom, the last layer was the top
//    digitalWrite(layerArray[lastAnode], HIGH); // turn off the previous layer **This is good
//    digitalWrite(layerArray[anodeLevel], LOW); // turn on the current layer  ** This is good
//      *portbyname[lastAnode] |=  _BV(layerbyname[lastAnode]); // This is faster x 2
//      *portbyname[anodeLevel] &= ~_BV(layerbyname[anodeLevel]); // This is faster
//    Note that the switch below tested out to be faster than the previous method due to array referencing. Faster x 5

   switch (lastAnode)
     {
    case 0:
      PORTG |= _BV(PG5);  // turn off the previous layer
      PORTE &= ~_BV(PE3); // turn on the current layer 
      break;
    case 1:
      PORTE |= _BV(PE3);  // turn off the previous layer
      PORTH &= ~_BV(PH3); // turn on the current layer 
      break;
    case 2:
      PORTH |= _BV(PH3);  // turn off the previous layer
      PORTH &= ~_BV(PH4); // turn on the current layer 
      break;
    case 3:
      PORTH |= _BV(PH4);  // turn off the previous layer
      PORTH &= ~_BV(PH5); // turn on the current layer 
      break;
    case 4:
      PORTH |= _BV(PH5);  // turn off the previous layer
      PORTH &= ~_BV(PH6); // turn on the current layer 
      break;
    case 5:
      PORTH |= _BV(PH6);  // turn off the previous layer
      PORTB &= ~_BV(PB4); // turn on the current layer 
      break;
    case 6:
      PORTB |= _BV(PB4);  // turn off the previous layer
      PORTB &= ~_BV(PB5); // turn on the current layer 
      break;
    case 7:
      PORTB |= _BV(PB5);  // turn off the previous layer
      PORTG &= ~_BV(PG5); // turn on the current layer 
      break;
     } 
         
PORTE |= 1<<latch_pin;//Latch pin HIGH
PORTE &= ~(1<<latch_pin);//Latch pin LOW
PORTE &= ~(1<<blank_pin);//Blank pin LOW to turn on the LEDs with the new data
// Blank is the same as the OE or ENABLE pin 

anodeLevel++;//increment the anode level
level = level+8;//increment the level variable by 8, which is used to shift out data, since the next level woudl be the next 8 bytes in the arrays

if(anodeLevel==8)//go back to 0 if max is reached
anodeLevel=0;
if(level==64)//if you hit 64 on level, this means you just sent out all 63 bytes, so go back
level=0;
pinMode(blank_pin, OUTPUT);//moved down here so outputs are all off until the first call of this function

// This next section is for SuperTech-IT's music module

if (MSGEQ7 == 1) 
   {
    if (digitalRead (A4) == LOW) {runMode = 0;} // animation button pressed - set animation mode
    if (digitalRead (A2) == LOW) {runMode++; }  // increase run mode by 1 if music mode button pressed
    while ((digitalRead (A4) == 0) || (digitalRead (A2) == 0)) {PORTE |= 1<<blank_pin;} // wait here until button is released

    if (runMode > modes) {runMode = 1;} // if we have gone through all the music modes, cycle back to sequenced mode
   }  // MSGEQ7 = 1

}//***MultiPlex BAM END***MultiPlex BAM END***MultiPlex BAM END***MultiPlex BAM END***MultiPlex BAM END***MultiPlex BAM END***MultiPlex BAM END



//*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE
//*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE
//*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE
//*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE*+*+*+*+*+*+*+*+*+*+*+*+PUT ANIMATIONS DOWN HERE



void fireworks (int iterations, int n, int delayx)
{
  fill(0,0,0);

  int i,f,e,x;

  float origin_x = 3;
  float origin_y = 3;
  float origin_z = 3;

  int rand_y, rand_x, rand_z;

  float slowrate, gravity;

  // Particles and their position, x,y,z and their movement, dx, dy, dz
  float particles[n][6];
  float lastpart[n][3];
  int inMode=runMode;

  while (iterations)
  {
    origin_x = rand()%4 + 2;
    origin_y = rand()%4 + 2;
    origin_z = rand()%2 + 5;

    // shoot a particle up in the air
    for (e=0;e<origin_z;e++)
    {
      if (runMode != inMode ) {return;} // drop out if mode changes
      LED (origin_x, origin_y, e,(random(16)),(random(16)),(random(16)));
      x=(50*e);
                delay(150);
      fill(0,0,0);
    }

    // Fill particle array
    for (f=0; f<n; f++)
    {
      // Position
      particles[f][0] = origin_x;
      particles[f][1] = origin_y;
      particles[f][2] = origin_z;
      
      rand_x = rand()%200;
      rand_y = rand()%200;
      rand_z = rand()%200;

      // Movement
      particles[f][3] = 1-(float)rand_x/100; // dx
      particles[f][4] = 1-(float)rand_y/100; // dy
      particles[f][5] = 1-(float)rand_z/100; // dz
    }

    // explode
    for (e=0; e<25; e++)
    {
      slowrate = 1+tan((e+0.1)/20)*10;
      gravity = tan((e+0.1)/20)/2;
      for (f=0; f<n; f++)
      { 
        particles[f][0] += particles[f][3]/slowrate;
        particles[f][1] += particles[f][4]/slowrate;
        particles[f][2] += particles[f][5]/slowrate;
        particles[f][2] -= gravity;     
        if (runMode != inMode ) {return;} // drop out if mode changes         
        LED (particles[f][0],particles[f][1], particles[f][2], (random(16)),(random(16)),(random(16)));
        lastpart[f][2]=particles[f][2];
        lastpart[f][0]=particles[f][0];
        lastpart[f][1]=particles[f][1];
      }

    delay(40);
        for (f=0; f<n; f++)
            {
            if (runMode != inMode ) {return;} // drop out if mode changes         
            LED (lastpart[f][0],lastpart[f][1],lastpart[f][2],0,0,0);      
            }
  }
  iterations--;
}
}


//**************************************************************************************************************************


void wipe_out(){//*****wipe_out*****wipe_out*****wipe_out*****wipe_out*****wipe_out*****wipe_out*****wipe_out*****wipe_out
  int xxx=0, yyy=0, zzz=0;
  int fx=random(8), fy=random(8), fz=random(8), direct, fxm=1, fym=1, fzm=1, fxo=0, fyo=0, fzo=0;
  int  ftx=random(8), fty=random(8), ftz=random(8), ftxm=1, ftym=1, ftzm=1, ftxo=0, ftyo=0, ftzo=0;
  int select, rr, gg, bb, rrt, ggt, bbt;
  int inMode=runMode;
  fill (0,0,0);
  //     ************************************  consider clean() above
       select=random(3);
    if(select==0){
      rr=random(1, 16);
      gg=random(1, 16);
      bb=0;} 
     if(select==1){
      rr=random(1, 16);
      gg=0;
      bb=random(1, 16);}    
     if(select==2){
      rr=0;
      gg=random(1, 16);
      bb=random(1, 16);}
      
       select=random(3);
    if(select==0){
      rrt=random(1, 16);
      ggt=random(1, 16);
      bbt=0;} 
     if(select==1){
      rrt=random(1, 16);
      ggt=0;
      bbt=random(1, 16);}    
     if(select==2){
      rrt=0;
      ggt=random(1, 16);
      bbt=random(1, 16);}  
        start=millis();
      
  while(millis()-start<10000){
    if (runMode != inMode ) {return;} // drop out if mode changes
    //fx=random(8); fy=random(8); fz=random(8);

    LED(fxo, fyo, fzo, 0, 0, 0);
    LED(fxo, fyo, fzo+1, 0, 0, 0);
    LED(fxo, fyo, fzo-1, 0, 0, 0);
    LED(fxo+1, fyo, fzo, 0, 0, 0);
    LED(fxo-1, fyo, fzo, 0, 0, 0);
    LED(fxo, fyo+1, fzo, 0, 0, 0);
    LED(fxo, fyo-1, fzo, 0, 0, 0);
    if (runMode != inMode ) {return;} // drop out if mode changes   
    LED(ftxo, ftyo, ftzo, 0, 0, 0);
    LED(ftxo, ftyo, ftzo+1, 0, 0, 0);
    LED(ftxo, ftyo, ftzo-1, 0, 0, 0);
    LED(ftxo+1, ftyo, ftzo, 0, 0, 0);
    LED(ftxo-1, ftyo, ftzo, 0, 0, 0);
    LED(ftxo, ftyo+1, ftzo, 0, 0, 0);
    LED(ftxo, ftyo-1, ftzo, 0, 0, 0);
    if (runMode != inMode ) {return;} // drop out if mode changes   
    LED(ftx, fty, ftz, rr, gg, bb);
    LED(ftx, fty, ftz+1, rr, gg, bb);
    LED(ftx, fty, ftz-1,  rr, gg, bb);
    LED(ftx+1, fty, ftz, rr, gg, bb);
    LED(ftx-1, fty, ftz, rr, gg, bb);
    LED(ftx, fty+1, ftz, rr, gg, bb);
    LED(ftx, fty-1, ftz, rr, gg, bb);     
    if (runMode != inMode ) {return;} // drop out if mode changes      
    LED(fx, fy, fz, rrt, ggt, bbt);
    LED(fx, fy, fz+1, rrt, ggt, bbt);
    LED(fx, fy, fz-1, rrt, ggt, bbt);
    LED(fx+1, fy, fz, rrt, ggt, bbt);
    LED(fx-1, fy, fz, rrt, ggt, bbt);
    LED(fx, fy+1, fz, rrt, ggt, bbt);
    LED(fx, fy-1, fz, rrt, ggt, bbt);  
    if (runMode != inMode ) {return;} // drop out if mode changes      
     delay(10);
     
    fxo=fx;
    fyo=fy;
    fzo=fz; 
    
    ftxo=ftx;
    ftyo=fty;
    ftzo=ftz; 
 
    direct=random(3);
    if(direct==0)
    fx= fx+fxm;
    if(direct==1)
    fy= fy+fym;  
    if(direct==2)
    fz= fz+fzm;  
  if(fx<0){
    fx=0; fxm=1;}
  if(fx>7){
    fx=7; fxm=-1;}  
  if(fy<0){
    fy=0; fym=1;}
  if(fy>7){
    fy=7; fym=-1;}    
  if(fz<0){
    fz=0; fzm=1;}
  if(fz>7){
    fz=7; fzm=-1;}  
  
      direct=random(3);
    if(direct==0)
    ftx= ftx+ftxm;
    if(direct==1)
    fty= fty+ftym;  
    if(direct==2)
    ftz= ftz+ftzm;  
  if(ftx<0){
    ftx=0; ftxm=1;}
  if(ftx>7){
    ftx=7; ftxm=-1;}  
  if(fty<0){
    fty=0; ftym=1;}
  if(fty>7){
    fty=7; ftym=-1;}    
  if(ftz<0){
    ftz=0; ftzm=1;}
  if(ftz>7){
    ftz=7; ftzm=-1;} 
  }//while
  fill (0,0,0);
 }//wipeout


void rainVersionTwo(){//****rainVersionTwo****rainVersionTwo****rainVersionTwo****rainVersionTwo****rainVersionTwo
  int x[64], y[64], z[64], addr, leds=64, bright=1, ledcolor, colowheel;
  int xx[64], yy[64], zz[64], xold[64], yold[64], zold[64], slowdown;
  int inMode=runMode;
  for(addr=0; addr<64; addr++){
    x[addr]=random(8);
    y[addr]=random(8);
    z[addr]=random(8);
    xx[addr]=random(16);
    yy[addr]=random(16);
    zz[addr]=random(16);     
  }
  start=millis();
  while(millis()-start<20000){
  //wipe_out();
  //for(addr=0; addr<leds; addr++)
  //LED( xold[addr], yold[addr], zold[addr], 0, 0, 0);
if (runMode != inMode ) {return;} // drop out if mode changes
if(ledcolor<200){
  for(addr=0; addr<leds; addr++){
    LED( xold[addr], yold[addr], zold[addr], 0, 0, 0);
  if(z[addr]>=7)
  LED( x[addr], y[addr], z[addr], 0, 5, 15);
  if(z[addr]==6)
  LED( x[addr], y[addr], z[addr], 0, 1, 9);
   if(z[addr]==5)
  LED( x[addr], y[addr], z[addr], 0, 0, 10);
   if(z[addr]==4)
  LED( x[addr], y[addr], z[addr], 1, 0, 11); 
    if(z[addr]==3)
  LED( x[addr], y[addr], z[addr], 3, 0, 12);
    if(z[addr]==2)
  LED( x[addr], y[addr], z[addr], 10, 0, 15);
    if(z[addr]==1)
  LED( x[addr], y[addr], z[addr], 10, 0, 10);
    if(z[addr]<=0)
  LED( x[addr], y[addr], z[addr], 10, 0, 1);
  if (runMode != inMode ) {return;} // drop out if mode changes   
}
}//200

  if(ledcolor>=200&&ledcolor<300){
  for(addr=0; addr<leds; addr++){
    LED( xold[addr], yold[addr], zold[addr], 0, 0, 0);
  if(z[addr]>=7)
  LED( x[addr], y[addr], z[addr], 15, 15, 0);
  if(z[addr]==6)
  LED( x[addr], y[addr], z[addr], 10, 10, 0);
   if(z[addr]==5)
  LED( x[addr], y[addr], z[addr], 15, 5, 0);
   if(z[addr]==4)
  LED( x[addr], y[addr], z[addr], 15, 2, 0); 
    if(z[addr]==3)
  LED( x[addr], y[addr], z[addr], 15, 1, 0);
    if(z[addr]==2)
  LED( x[addr], y[addr], z[addr], 15, 0, 0);
    if(z[addr]==1)
  LED( x[addr], y[addr], z[addr], 12, 0, 0);
    if(z[addr]<=0)
  LED( x[addr], y[addr], z[addr], 10, 0, 0);
  if (runMode != inMode ) {return;} // drop out if mode changes   
}}//300

    if(ledcolor>=300&&ledcolor<400){

  }
     if(ledcolor>=500&&ledcolor<600){

}
  
  
  ledcolor++;
if(ledcolor>=300)
ledcolor=0;
  
    for(addr=0; addr<leds; addr++){
    xold[addr]=x[addr];
    yold[addr]=y[addr];
    zold[addr]=z[addr];
   } 
  
  
  delay(15);
  
  //for(addr=0; addr<leds; addr++)
  //LED( x[addr], y[addr], z[addr], 0, 0, 0);
    for(addr=0; addr<leds; addr++){
      
      //slowdown = random(2);
      //if(bitRead(z[addr],0))
  z[addr] = z[addr]-1;
  
 // x[addr] = x[addr]+1;
 // y[addr] = y[addr]+1;
  if(z[addr]<random(-100,0)){
    x[addr]=random(8);
    y[addr]=random(8);
    int select=random(3);
    if(select==0){
    xx[addr]=0;
    zz[addr]=random(16);
    yy[addr]=random(16);
    //zz[addr]=0;
    }
    if(select==1){
    xx[addr]=random(16);
    zz[addr]=0;
    yy[addr]=random(16);
    //yy[addr]=0;
    }
    if(select==2){
    xx[addr]=random(16);
    zz[addr]=random(16);
    yy[addr]=0;
    
    
    }    
   z[addr]=7; 
   
  }//-check
  }//add

  
  }//while
 
}//rainv2


void folder(){//****folder****folder****folder****folder****folder****folder****folder****folder****folder
  int xx, yy, zz, pullback[16], state=0, backorfront=7;//backorfront 7 for back 0 for front
  
  int folderaddr[16], LED_Old[16], oldpullback[16], ranx=random(16), rany=random(16), ranz=random(16), ranselect;
  int bot=0, top=1, right=0, left=0, back=0, front=0, side=0, side_select;
  int inMode=runMode;  
  folderaddr[0]=-7;
  folderaddr[1]=-6;
  folderaddr[2]=-5;
  folderaddr[3]=-4;
  folderaddr[4]=-3;
  folderaddr[5]=-2;
  folderaddr[6]=-1;
  folderaddr[7]=0;
  
  for(xx=0; xx<8; xx++){
  oldpullback[xx]=0;
  pullback[xx]=0;
  }
  
  start=millis();
  while(millis()-start<10000){ 
  if (runMode != inMode ) {return;} // drop out if mode changes
  if(top==1){
      if(side==0){
   //top to left-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes   
  LED(7-LED_Old[yy], yy-oldpullback[yy],xx , 0, 0, 0);
  LED(7-folderaddr[yy], yy-pullback[yy],xx , ranx, rany, ranz);
  }}}
    if(side==2){
      //top to back-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes   
  LED(7-LED_Old[yy], xx, yy-oldpullback[yy], 0, 0, 0);
  LED(7-folderaddr[yy], xx, yy-pullback[yy], ranx, rany, ranz);
  }}}
  if(side==3){
      //top-side to front-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes   
  LED(7-LED_Old[7-yy], xx, yy+oldpullback[yy], 0, 0, 0);
  LED(7-folderaddr[7-yy], xx, yy+pullback[yy], ranx, rany, ranz);
  }}}
    if(side==1){
    //top-side to right
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(7-LED_Old[7-yy], yy+oldpullback[yy],xx , 0, 0, 0);
  LED(7-folderaddr[7-yy], yy+pullback[yy],xx , ranx, rany, ranz);
  }}}
    }//top
    
    if(right==1){
    if(side==4){
   //right-side to top
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy+oldpullback[7-yy],7-LED_Old[7-yy],xx , 0, 0, 0);
  LED( yy+pullback[7-yy],7-folderaddr[7-yy],xx , ranx, rany, ranz);
  }}}
    if(side==3){
    //right-side to front-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, 7-LED_Old[7-yy],yy+oldpullback[yy], 0, 0, 0);
  LED(xx,7-folderaddr[7-yy], yy+pullback[yy], ranx, rany, ranz);
  }}}
    if(side==2){
   //right-side to back-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, 7-LED_Old[yy],yy-oldpullback[yy], 0, 0, 0);
  LED(xx,7-folderaddr[yy], yy-pullback[yy], ranx, rany, ranz);
  }}}
    if(side==5){
    //right-side to bottom
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy-oldpullback[yy],7-LED_Old[yy],xx , 0, 0, 0);
  LED( yy-pullback[yy],7-folderaddr[yy],xx , ranx, rany, ranz);
  }}}
    }//right
    
 if(left==1){
    if(side==4){
   //left-side to top
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy+oldpullback[yy],LED_Old[7-yy],xx , 0, 0, 0);
  LED( yy+pullback[yy],folderaddr[7-yy],xx , ranx, rany, ranz);
  }}}
  if(side==3){
     //left-side to front-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, LED_Old[7-yy],yy+oldpullback[yy], 0, 0, 0);
  LED(xx,folderaddr[7-yy], yy+pullback[yy], ranx, rany, ranz);
  }}}
  if(side==2){
//left-side to back-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, LED_Old[yy],yy-oldpullback[yy], 0, 0, 0);
  LED(xx,folderaddr[yy], yy-pullback[yy], ranx, rany, ranz);
  }}}
  if(side==5){
  //left-side to bottom
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy-oldpullback[yy],LED_Old[yy],xx , 0, 0, 0);
  LED( yy-pullback[yy],folderaddr[yy],xx , ranx, rany, ranz);
  }}}
  }//left
  
   if(back==1){
    if(side==1){
  //back-side to right-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, yy+oldpullback[yy],LED_Old[7-yy], 0, 0, 0);
  LED(xx, yy+pullback[yy],folderaddr[7-yy], ranx, rany, ranz);
  }}}
  if(side==4){
  // back-side to top-side
    for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy+oldpullback[yy],xx,LED_Old[7-yy] , 0, 0, 0);
  LED( yy+pullback[yy],xx,folderaddr[7-yy] , ranx, rany, ranz);
  }}}
  if(side==5){
  // back-side to bottom
    for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy-oldpullback[yy],xx,LED_Old[yy] , 0, 0, 0);
  LED( yy-pullback[yy],xx,folderaddr[yy] , ranx, rany, ranz);
  }}}//state1
 if(side==0){
//back-side to left-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, yy-oldpullback[yy],LED_Old[yy], 0, 0, 0);
  LED(xx, yy-pullback[yy],folderaddr[yy], ranx, rany, ranz);
  }}}
   }//back
   if(bot==1){
    if(side==1){
    // bottom-side to right-side
      for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(LED_Old[7-yy], yy+oldpullback[yy],xx , 0, 0, 0);
  LED(folderaddr[7-yy], yy+pullback[yy],xx , ranx, rany, ranz);
  }}}
  if(side==3){
  //bottom to front-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(LED_Old[7-yy], xx, yy+oldpullback[yy], 0, 0, 0);
  LED(folderaddr[7-yy], xx, yy+pullback[yy], ranx, rany, ranz);
  }}}
if(side==2){
  //bottom to back-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(LED_Old[yy], xx, yy-oldpullback[yy], 0, 0, 0);
  LED(folderaddr[yy], xx, yy-pullback[yy], ranx, rany, ranz);
  }}}
if(side==0){
  //bottom to left-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(LED_Old[yy], yy-oldpullback[yy],xx , 0, 0, 0);
  LED(folderaddr[yy], yy-pullback[yy],xx , ranx, rany, ranz);
  }}}
   }//bot
   
    if(front==1){
   if(side==0){
    //front-side to left-side
   for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, yy-oldpullback[yy],7-LED_Old[yy], 0, 0, 0);
  LED(xx, yy-pullback[yy],7-folderaddr[yy], ranx, rany, ranz);
  }}}
  if(side==5){
      // front-side to bottom
    for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy-oldpullback[yy],xx,7-LED_Old[yy] , 0, 0, 0);
  LED( yy-pullback[yy],xx,7-folderaddr[yy] , ranx, rany, ranz);
  }}}
  if(side==4){
      // front-side to top-side
    for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(yy+oldpullback[yy],xx,7-LED_Old[7-yy] , 0, 0, 0);
  LED( yy+pullback[yy],xx,7-folderaddr[7-yy] , ranx, rany, ranz);
  }}}
  if(side==1){
      //front-side to right-side
  for(yy=0; yy<8; yy++){
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED(xx, yy+oldpullback[yy],7-LED_Old[7-yy], 0, 0, 0);
  LED(xx, yy+pullback[yy],7-folderaddr[7-yy], ranx, rany, ranz);
  }}}
    }//front

  delay(5);//               DELAY   DELAY  DELAY
  for(xx=0; xx<8; xx++){
  if (runMode != inMode ) {return;} // drop out if mode changes     
  LED_Old[xx]=folderaddr[xx];
  oldpullback[xx]=pullback[xx];
  }
  
  if(folderaddr[7]==7){
   // pullback=8;
    for(zz=0; zz<8; zz++)
  pullback[zz] = pullback[zz]+1;
 
   if(pullback[7]==8){//finished with fold
   delay(10);
     //state++;
     //if(state==4)
     //state=0;
     
     ranselect= random(3);
     if(ranselect==0){
     ranx=0;
     rany=random(1,16);
     ranz=random(1,16);}
     if(ranselect==1){
     ranx=random(1,16);
     rany=0;
     ranz=random(1,16);}
     if(ranselect==2){
     ranx=random(1,16);
     rany=random(1,16);
     ranz=0;}     
     
     side_select=random(3);
     
     if(top==1){//                 TOP
     top=0; 
     if(side==0){//top to left
      left=1;
      if(side_select==0) side=2;
      if(side_select==1) side=3;
      //if(side_select==2) side=4;
      if(side_select==2) side=5;} else    
     if(side==1){//top to right
      right=1;
      if(side_select==0) side=5;
      if(side_select==1) side=2;
      if(side_select==2) side=3;
      //if(side_select==3) side=4;
    } else  
     if(side==2){//top to back
      back=1;
      if(side_select==0) side=0;
      if(side_select==1) side=1;
      if(side_select==2) side=5;
      //if(side_select==3) side=4;
    } else      
      if(side==3){//top to front
      front=1;
      if(side_select==0) side=0;
      if(side_select==1) side=1;
      if(side_select==2) side=5;
      //if(side_select==3) side=4;
    }   
     } else//top
     if(bot==1){//                 BOTTOM
     bot=0; 
     if(side==0){//bot to left
      left=1;
      if(side_select==0) side=2;
      if(side_select==1) side=3;
      if(side_select==2) side=4;
      //if(side_select==3) side=5;
    } else    
     if(side==1){//bot to right
      right=1;
      //if(side_select==0) side=5;
      if(side_select==0) side=2;
      if(side_select==1) side=3;
      if(side_select==2) side=4;} else  
     if(side==2){//bot to back
      back=1;
      if(side_select==0) side=0;
      if(side_select==1) side=1;
      //if(side_select==2) side=5;
      if(side_select==2) side=4;} else      
      if(side==3){//bot to front
      front=1;
      if(side_select==0) side=0;
      if(side_select==1) side=1;
      //if(side_select==2) side=5;
      if(side_select==2) side=4;}   
     } else//bot
          if(right==1){//                 RIGHT
     right=0; 
     if(side==4){//right to top
      top=1;
      if(side_select==0) side=2;
      if(side_select==1) side=3;
      if(side_select==2) side=0;
      //if(side_select==3) side=1;
    } else    
     if(side==5){//right to bot
      bot=1;
      if(side_select==0) side=0;
      if(side_select==1) side=2;
      if(side_select==2) side=3;
      //if(side_select==3) side=1;
    } 
      else  
     if(side==2){//right to back
      back=1;
      if(side_select==0) side=0;
      //if(side_select==1) side=1;
      if(side_select==1) side=5;
      if(side_select==2) side=4;} else      
      if(side==3){//right to front
      front=1;
      if(side_select==0) side=0;
      //if(side_select==1) side=1;
      if(side_select==1) side=5;
      if(side_select==2) side=4;}   
     } else//bot
              if(left==1){//                 LEFT
     left=0; 
     if(side==4){//left to top
      top=1;
      //if(side_select==0) side=2;
      if(side_select==0) side=3;
      if(side_select==1) side=2;
      if(side_select==2) side=1;} else    
     if(side==5){//left to bot
      bot=1;
      //if(side_select==0) side=0;
      if(side_select==0) side=2;
      if(side_select==1) side=3;
      if(side_select==2) side=1;} else  
     if(side==2){//left to back
      back=1;
      //if(side_select==0) side=0;
      if(side_select==0) side=1;
      if(side_select==1) side=5;
      if(side_select==2) side=4;} else      
      if(side==3){//left to front
      front=1;
      //if(side_select==0) side=0;
      if(side_select==0) side=1;
      if(side_select==1) side=5;
      if(side_select==2) side=4;}   
     } else//bot
              if(front==1){//                 front
     front=0; 
     if(side==4){//front to top
      top=1;
      if(side_select==0) side=2;
      //if(side_select==1) side=3;
      if(side_select==1) side=0;
      if(side_select==2) side=1;} else    
     if(side==5){//front to bot
      bot=1;
      if(side_select==0) side=0;
      if(side_select==1) side=2;
      //if(side_select==2) side=3;
      if(side_select==2) side=1;} else  
     if(side==0){//front to left
      left=1;
      if(side_select==0) side=2;
     // if(side_select==1) side=3;
      if(side_select==1) side=5;
      if(side_select==2) side=4;} else      
      if(side==1){//front to right
      right=1;
      if(side_select==0) side=2;
     // if(side_select==1) side=3;
      if(side_select==1) side=5;
      if(side_select==2) side=4;}   
     } else//bot
              if(back==1){//                 back
     back=0; 
     if(side==4){//back to top
      top=1;
      //if(side_select==0) side=2;
      if(side_select==0) side=3;
      if(side_select==1) side=0;
      if(side_select==2) side=1;} else    
     if(side==5){//back to bot
      bot=1;
      if(side_select==0) side=0;
      //if(side_select==1) side=2;
      if(side_select==1) side=3;
      if(side_select==2) side=1;} else  
     if(side==0){//back to left
      left=1;
      //if(side_select==0) side=2;
      if(side_select==0) side=3;
      if(side_select==1) side=5;
      if(side_select==2) side=4;} else      
      if(side==1){//back to right
      right=1;
      //if(side_select==0) side=2;
      if(side_select==0) side=3;
      if(side_select==1) side=5;
      if(side_select==2) side=4;}   
     } //bot

    
      // for(yy=0; yy<8; yy++)
  //for(xx=0; xx<8; xx++)
  //LED(LED_Old[yy], xx, yy-oldpullback[yy], 0, 0, 0);
  for(xx=0; xx<8; xx++){
  oldpullback[xx]=0;
  pullback[xx]=0;}
 
  folderaddr[0]=-8;
  folderaddr[1]=-7;
  folderaddr[2]=-6;
  folderaddr[3]=-5;
  folderaddr[4]=-4;
  folderaddr[5]=-3;
  folderaddr[6]=-2;
  folderaddr[7]=-1;

}//pullback==7
  }//folderaddr==7    

  if(folderaddr[7]!=7)
  for(zz=0; zz<8; zz++)
  folderaddr[zz] = folderaddr[zz]+1;
  
}//while

}//folder


void bouncyvTwo(){//****bouncyTwo****bouncyTwo****bouncyTwo****bouncyTwo****bouncyTwo****bouncyTwo****bouncyTwo
  int wipex, wipey, wipez, ranr, rang, ranb, select, oldx[50], oldy[50], oldz[50];
  int x[50], y[50], z[50], addr, ledcount=20, direct, direcTwo;
  int xx[50], yy[50], zz[50];
  int xbit=1, ybit=1, zbit=1;
  int inMode=runMode;
   for(addr=0; addr<ledcount+1; addr++){
     oldx[addr]=0;
     oldy[addr]=0;
     oldz[addr]=0;
     x[addr]=0;
     y[addr]=0;
     z[addr]=0;
     xx[addr]=0;
     yy[addr]=0;
     zz[addr]=0;
    }
  
  start=millis();
      
  while(millis()-start<15000){
  if (runMode != inMode ) {return;} // drop out if mode changes
  direct = random(3);

for(addr=1; addr<ledcount+1; addr++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED(oldx[addr], oldy[addr],oldz[addr], 0,0,0);
LED(x[addr], y[addr], z[addr], xx[addr],yy[addr],zz[addr]);
}

for(addr=1; addr<ledcount+1; addr++){
oldx[addr]=x[addr];
oldy[addr]=y[addr];
oldz[addr]=z[addr];
}
delay(20);


//direcTwo=random(3);  
//if(direcTwo==1)



if(direct==0)
x[0]= x[0]+xbit;
if(direct==1)
y[0]= y[0]+ybit;
if(direct==2)
z[0]= z[0]+zbit;

if(direct==3)
x[0]= x[0]-xbit;
if(direct==4)
y[0]= y[0]-ybit;
if(direct==5)
z[0]= z[0]-zbit;

if(x[0]>7){
xbit=-1;
x[0]=7;
xx[0]=random(16);
yy[0]=random(16);
zz[0]=0;
//wipe_out();
}
if(x[0]<0){
xbit=1;
  x[0]=0;
xx[0]=random(16);
yy[0]=0;
zz[0]=random(16);
//wipe_out();
}
if(y[0]>7){
ybit=-1;
y[0]=7;
xx[0]=0;
yy[0]=random(16);
zz[0]=random(16);
//wipe_out();
}
if(y[0]<0){
ybit=1;
  y[0]=0;
  xx[0]=0;
yy[0]=random(16);
zz[0]=random(16);
//wipe_out();
}
if(z[0]>7){
zbit=-1;
z[0]=7;
xx[0]=random(16);
yy[0]=0;
zz[0]=random(16);
//wipe_out();
}
if(z[0]<0){
zbit=1;
  z[0]=0;
  xx[0]=random(16);
yy[0]=random(16);
zz[0]=0;
//wipe_out();
}

for(addr=ledcount; addr>0; addr--){
  x[addr]=x[addr-1];
  y[addr]=y[addr-1];
  z[addr]=z[addr-1];
  xx[addr]=xx[addr-1];
  yy[addr]=yy[addr-1];
  zz[addr]=zz[addr-1];
}
}//while
  
}//bouncyv2


void sinwaveTwo(){//*****sinewaveTwo*****sinewaveTwo*****sinewaveTwo*****sinewaveTwo*****sinewaveTwo*****sinewaveTwo
  int sinewavearray[8], addr, sinemult[8], colselect, rr=0, gg=0, bb=15, addrt;
  int sinewavearrayOLD[8], select, subZ=-7, subT=7, multi=0;//random(-1, 2);
  int inMode=runMode;
  sinewavearray[0]=0;
  sinemult[0]=1;
   sinewavearray[1]=1;
  sinemult[1]=1; 
    sinewavearray[2]=2;
  sinemult[2]=1;
    sinewavearray[3]=3;
  sinemult[3]=1;
    sinewavearray[4]=4;
  sinemult[4]=1;
    sinewavearray[5]=5;
  sinemult[5]=1;
    sinewavearray[6]=6;
  sinemult[6]=1;
    sinewavearray[7]=7;
  sinemult[7]=1;
  
  start=millis();
      
  while(millis()-start<15000){
  if (runMode != inMode ) {return;} // drop out if mode changes
  for(addr=0; addr<8; addr++){
    if(sinewavearray[addr]==7){
    sinemult[addr]=-1;
    }
    if(sinewavearray[addr]==0){
    sinemult[addr]=1;     
    }
    sinewavearray[addr] = sinewavearray[addr] + sinemult[addr];
}//addr
     if(sinewavearray[0]==7){
     select=random(3);
    if(select==0){
      rr=random(1, 16);
      gg=random(1, 16);
      bb=0;} 
     if(select==1){
      rr=random(1, 16);
      gg=0;
      bb=random(1, 16);}    
     if(select==2){
      rr=0;
      gg=random(1, 16);
      bb=random(1, 16);}
   /*
 if(multi==1)
 multi=0;
 else
 multi=1;
*/

 }    

 for(addr=0; addr<8; addr++){
   if (runMode != inMode ) {return;} // drop out if mode changes 
   LED( addr, 0, sinewavearrayOLD[addr], 0, 0, 0);
   LED( 0, addr, sinewavearrayOLD[addr], 0, 0, 0);
   LED( subT-addr, 7, sinewavearrayOLD[addr], 0, 0, 0);
   LED( 7, subT-addr, sinewavearrayOLD[addr], 0, 0, 0);     
   LED( addr, 0, sinewavearray[addr], rr, gg, bb);
   LED( 0, addr, sinewavearray[addr], rr, gg, bb);
   LED( subT-addr, 7, sinewavearray[addr], rr, gg, bb);
   LED( 7, subT-addr, sinewavearray[addr], rr, gg, bb);
    }
    
       for(addr=1; addr<7; addr++){   
   if (runMode != inMode ) {return;} // drop out if mode changes         
    LED( addr, 1, sinewavearrayOLD[addr+multi*1], 0, 0, 0);
    LED( 1, addr, sinewavearrayOLD[addr+multi*1], 0, 0, 0);
    LED( subT-addr, 6, sinewavearrayOLD[addr+multi*1], 0, 0, 0);
    LED( 6, subT-addr, sinewavearrayOLD[addr+multi*1], 0, 0, 0);  
    LED( addr, 1, sinewavearray[addr+multi*1], rr, gg, bb);
    LED( 1, addr, sinewavearray[addr+multi*1], rr, gg, bb);
    LED( subT-addr,6, sinewavearray[addr+multi*1], rr, gg, bb);
    LED( 6, subT-addr, sinewavearray[addr+multi*1], rr, gg, bb);
       }
 
        for(addr=2; addr<6; addr++){   
  if (runMode != inMode ) {return;} // drop out if mode changes           
    LED( addr, 2, sinewavearrayOLD[addr+multi*2], 0, 0, 0);
    LED( 2, addr, sinewavearrayOLD[addr+multi*2], 0, 0, 0);
    LED( subT-addr, 5, sinewavearrayOLD[addr+multi*2], 0, 0, 0);
    LED( 5, subT-addr, sinewavearrayOLD[addr+multi*2], 0, 0, 0);  
    LED( addr, 2, sinewavearray[addr+multi*2], rr, gg, bb);
    LED( 2, addr, sinewavearray[addr+multi*2], rr, gg, bb);
    LED( subT-addr, 5, sinewavearray[addr+multi*2], rr, gg, bb);
    LED( 5, subT-addr, sinewavearray[addr+multi*2], rr, gg, bb);
       }  
             for(addr=3; addr<5; addr++){   
   if (runMode != inMode ) {return;} // drop out if mode changes               
    LED( addr, 3, sinewavearrayOLD[addr+multi*3], 0, 0, 0);
    LED( 3, addr, sinewavearrayOLD[addr+multi*3], 0, 0, 0);
    LED( subT-addr, 4, sinewavearrayOLD[addr+multi*3], 0, 0, 0);
    LED( 4, subT-addr, sinewavearrayOLD[addr+multi*3], 0, 0, 0);  
   LED( addr, 3, sinewavearray[addr+multi*3], rr, gg, bb);
   LED( 3, addr, sinewavearray[addr+multi*3], rr, gg, bb);
   LED( subT-addr,4, sinewavearray[addr+multi*3], rr, gg, bb);
   LED( 4, subT-addr, sinewavearray[addr+multi*3], rr, gg, bb);
       }      
     
     for(addr=0; addr<8; addr++)
   sinewavearrayOLD[addr]=sinewavearray[addr];
   if (runMode != inMode ) {return;} // drop out if mode changes 
    delay(30);
   
  }//while
  
  
}//SinewaveTwo

void color_wheel(){
  int xx, yy, zz, ww, rr=1, gg=1, bb=1, ranx, rany, swiper;
  int inMode=runMode;
  
  start=millis();
      
  while(millis()-start<100000){

    swiper=random(3);
    ranx=random(16);
    rany=random(16);
     
    for(xx=0;xx<8;xx++){
    for(yy=0;yy<8;yy++){ 
    for(zz=0;zz<8;zz++){
     if (runMode != inMode ) {return;} // drop out if mode changes      
     LED(xx, yy, zz,  ranx, 0, rany);
    }}
  delay(50);
}

     ranx=random(16);
     rany=random(16);
     
    for(xx=7;xx>=0;xx--){ 
    for(yy=0;yy<8;yy++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes
    LED(xx,yy, zz, ranx, rany, 0);
    }}
  delay(50); 
  }
       ranx=random(16);
     rany=random(16);
    for(xx=0;xx<8;xx++){ 
    for(yy=0;yy<8;yy++){
    for(zz=0;zz<8;zz++){
      if (runMode != inMode ) {return;} // drop out if mode changes      
      LED(xx,yy, zz, 0, ranx, rany);
    }}
    delay(50);
  }
    
     ranx=random(16);
     rany=random(16);
    for(xx=7;xx>=0;xx--){ 
    for(yy=0;yy<8;yy++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes      
    LED(xx,yy, zz, rany, ranx, 0);
    }}
  delay(50); 
  }
    
  }//while
    
}//color wheel

void color_wheelTWO(){//*****colorWheelTwo*****colorWheelTwo*****colorWheelTwo*****colorWheelTwo*****colorWheelTwo
  int xx, yy, zz, ww, rr=1, gg=1, bb=1, ranx, rany ,ranz, select, swiper;
  int inMode=runMode;

  start=millis();
  while(millis()-start<10000){
    if (runMode != inMode ) {return;} // drop out if mode changes
    swiper=random(6);
    select=random(3);
    if(select==0){
     ranx=0;
     rany=random(16);
     ranz=random(16);}
    if(select==1){
     ranx=random(16);
     rany=0;
     ranz=random(16);}   
      if(select==2){
     ranx=random(16);
     rany=random(16);
     ranz=0;}  
    
     
    if(swiper==0){
    for(yy=0;yy<8;yy++){//left to right
    for(xx=0;xx<8;xx++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
  delay(30);
 }}
    if(swiper==1){//bot to top
    for(xx=0;xx<8;xx++){
    for(yy=0;yy<8;yy++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
  delay(30);
 }}  
    if(swiper==2){//back to front
    for(zz=0;zz<8;zz++){
    for(xx=0;xx<8;xx++){
    for(yy=0;yy<8;yy++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
  delay(30);
}}    
    if(swiper==3){
    for(yy=7;yy>=0;yy--){//right to left
    for(xx=0;xx<8;xx++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
  delay(30);
}}
    if(swiper==4){//top to bot
    for(xx=7;xx>=0;xx--){
    for(yy=0;yy<8;yy++){
    for(zz=0;zz<8;zz++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
 delay(30);
}}  
    if(swiper==5){//front to back
    for(zz=7;zz>=0;zz--){
    for(xx=0;xx<8;xx++){
    for(yy=0;yy<8;yy++){
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(xx, yy, zz,  ranx, ranz, rany);
    }}
  delay(30);
}}
  
}//while
}//color wheel

//    Harlem Shake    Harlem Shake    Harlem Shake    Harlem Shake    Harlem Shake
void harlem_shake()
{ 
int greenx = random(1,7),greeny = random(1,7),bluex = random(1,7),bluey = random(1,7),redx = random(1,7),redy = random(1,7);
int greenmult=1, bluemult=1, redmult=1;
int greenmulty=1, bluemulty=1, redmulty=1;
int oredx, oredy,obluex,obluey,ogreenx,ogreeny, cb1=15,cb2=0,cr1=15,cr2=0,cg1=15,cg2=0;
int time_counter=10, timemult=2;
int m;
int c1=1,c2=1,c3=1,xmult=1,ymult=1,zmult=1,x=4,y=4,z=4,color_select, xo, yo, zo;
int c21=1,c22=1,c23=1,x2mult=1,y2mult=1,z2mult=1,x2=2,y2=2,z2=2,color_select2, x2o, y2o, z2o;
int counter, i, j, k;
int inMode=runMode;

for(counter=0; counter<150; counter++){
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes
  LED(oredx,oredx,i,0,0,0);
  }
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes    
  LED(redx,redx,i,15,0,0);
  }
  
  oredx=redx;
  oredy=redy;
  
for(i=100; i>time_counter; i--)
  {
  if (runMode != inMode ) {return;} // drop out if mode changes
  delay(1);
  }


  time_counter=time_counter+timemult;
  if(time_counter>100||time_counter<10)
  timemult=timemult*-1;


   if(redy>6 || redy<1){
  redmulty=redmulty*-1;  
  
  }
 
   if(redx>6 || redx<1){
  redmult=redmult*-1;  
  
  cr1=random(16);
  cr2=random(16);
   }

  redy=redy+redmulty;
  redx=redx+redmult;
}//counter


for(counter=0; counter<85; counter++){
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(oredx,oredx,i,0,0,0);
  LED(i,ogreeny,ogreenx,0,0,0);
  }
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(redx,redx,i,15,0,0);
  LED(i,greeny,greenx,0,15,0);
  }
  ogreenx=greenx;
  ogreeny=greeny;
  oredx=redx;
  oredy=redy;
  
for(i=100; i>time_counter; i--)
  delay(1);

  time_counter=time_counter+timemult;
  if(time_counter>100||time_counter<10)
  timemult=timemult*-1;

   
     if(greeny>6 || greeny<1)
  greenmulty=greenmulty*-1;

   if(redy>6 || redy<1){
  redmulty=redmulty*-1;  
  
  }
 
   
     if(greenx>6 || greenx<1){
  greenmult=greenmult*-1;
  greeny = greeny + greenmulty;
  cg1=random(16);
  cg2=random(16);
  }

   if(redx>6 || redx<1){
  redmult=redmult*-1;  
  
  cr1=random(16);
  cr2=random(16);
   }
  greenx=greenx+greenmult;
  
  redy=redy+redmulty;
  redx=redx+redmult;
}//counter


for(counter=0; counter<85; counter++){
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(oredx,oredx,i,0,0,0);
  LED(obluex,i,obluey,0,0,0);
  LED(i,ogreeny,ogreenx,0,0,0);
  }
  for(i=0; i<8; i++){
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(redx,redx,i,15,0,0);
  LED(bluex,i,bluey,0,0,15);
  LED(i,greeny,greenx,0,15,0);
  }
  ogreenx=greenx;
  ogreeny=greeny;
  obluex=bluex;
  obluey=bluey;
  oredx=redx;
  oredy=redy;
  
for(i=100; i>time_counter; i--)
  delay(1);

  time_counter=time_counter+timemult;
  if(time_counter>100||time_counter<10)
  timemult=timemult*-1;

   
     if(greeny>6 || greeny<1)
  greenmulty=greenmulty*-1;

   if(bluey>6 || bluey<1)
  bluemulty=bluemulty*-1;

   if(redy>6 || redy<1){
  redmulty=redmulty*-1;  
  
  }
 
  if(greenx>6 || greenx<1){
  greenmult=greenmult*-1;
  greeny = greeny + greenmulty;
  cg1=random(16);
  cg2=random(16);
  }
  if(bluex>6 || bluex<1){
  bluemult=bluemult*-1;
  bluey=bluey+bluemulty;
  cb1=random(16);
  cb2=random(16);
   }
   if(redx>6 || redx<1){
  redmult=redmult*-1;  
  
  cr1=random(16);
  cr2=random(16);
   }
  greenx=greenx+greenmult;
  bluex=bluex+bluemult;
  redy=redy+redmulty;
  redx=redx+redmult;
}//counter


for(counter=0; counter<3; counter++){ // counter was 3
  for(i=0; i<8; i++)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
    {
    if (runMode != inMode ) {return;} // drop out if mode changes 
    LED(j,k,i,8,15,15);// red too intense to make white at 15. Reduced to 8 - makes nice white level.
    }
  delay(50);
  for(i=0; i<8; i++)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
    {
    if (runMode != inMode ) {return;} // drop out if mode changes  
    LED(j,k,i,0,0,0);
    }
  delay(50);
}//counter

  for(m=0; m<1; m++){
    
    
  for(i=0; i<8; i++)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
  {
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(j,k,i,0,random(16),random(16));
  }
  for(i=7; i>=0; i--)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
  {
   if (runMode != inMode ) {return;} // drop out if mode changes 
   LED(j,k,i,random(16),0,random(16));
  }

  for(i=0; i<8; i++)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
      {
      if (runMode != inMode ) {return;} // drop out if mode changes 
      LED(j,k,i,random(16),random(16),0);
      }

  for(i=7; i>=0; i--)
  for(j=0; j<8; j++)
  for(k=0; k<8; k++)
  {
  if (runMode != inMode ) {return;} // drop out if mode changes 
  LED(j,k,i,random(16),0,random(16));
  }
  }

fill(0,0,0); 

// this is the random changing led colour full cube

for(m=0; m<20; m++){
    for(k=0; k<200; k++){
  if (runMode != inMode ) {return;} // drop out if mode changes
  LED(random(8),random(8),random(8),random(16),random(16),0);
  LED(random(8),random(8),random(8),random(16),0 ,random(16));
  LED(random(8),random(8),random(8),0, random(16),random(16));
   //   c1=random(8);
     // c2=random(8);
      //c3=random(8);
  //LED(c1,c2,c3,15,15,15);
  }
      for(k=0; k<200; k++){
      if (runMode != inMode ) {return;} // drop out if mode changes         
      LED(random(8),random(8),random(8),0,0,0);
      }
  
}//m
  
fill(0,0,0);

color_select=random(0,3);
if(color_select==0){
c1 = 0;
c2 = random(0,16);
c3 = random(0,16);}
if(color_select==1){
c1 = random(0,16);
c2 = 0;
c3 = random(0,16);}
if(color_select==2){
c1 = random(0,16);
c2 = random(0,16);
c3 = 0;}


color_select2=random(0,3);
if(color_select2==0){
c21 = 0;
c22 = random(0,16);
c23 = random(0,16);}
if(color_select2==1){
c21 = random(0,16);
c22 = 0;
c23 = random(0,16);}
if(color_select2==2){
c21 = random(0,16);
c22 = random(0,16);
c23 = 0;}

 for(counter=0; counter<200; counter++){
if (runMode != inMode ) {return;} // drop out if mode changes
LED(xo,yo,zo,0,0,0);
LED(xo+1,yo,zo,0,0,0);
LED(xo+2,yo,zo,0,0,0);
LED(xo-1,yo,zo,0,0,0);
LED(xo-2,yo,zo,0,0,0);
LED(xo,yo+1,zo,0,0,0);
LED(xo,yo-1,zo,0,0,0);
LED(xo,yo+2,zo,0,0,0);
LED(xo,yo-2,zo,0,0,0);
LED(xo,yo,zo-1,0,0,0);
LED(xo,yo,zo+1,0,0,0);
LED(xo,yo,zo-2,0,0,0);
LED(xo,yo,zo+2,0,0,0);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x2o,y2o,z2o,0,0,0);
LED(x2o+1,y2o,z2o,0,0,0);
LED(x2o+2,y2o,z2o,0,0,0);
LED(x2o-1,y2o,z2o,0,0,0);
LED(x2o-2,y2o,z2o,0,0,0);
LED(x2o,y2o+1,z2o,0,0,0);
LED(x2o,y2o-1,z2o,0,0,0);
LED(x2o,y2o+2,z2o,0,0,0);
LED(x2o,y2o-2,z2o,0,0,0);
LED(x2o,y2o,z2o-1,0,0,0);
LED(x2o,y2o,z2o+1,0,0,0);
LED(x2o,y2o,z2o-2,0,0,0);
LED(x2o,y2o,z2o+2,0,0,0);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(xo+1,yo+1,zo,0,0,0);
LED(xo+1,yo-1,zo,0,0,0);
LED(xo-1,yo+1,zo,0,0,0);
LED(xo-1,yo-1,zo,0,0,0);
LED(xo+1,yo+1,zo+1,0,0,0);
LED(xo+1,yo-1,zo+1,0,0,0);
LED(xo-1,yo+1,zo+1,0,0,0);
LED(xo-1,yo-1,zo+1,0,0,0);
LED(xo+1,yo+1,zo-1,0,0,0);
LED(xo+1,yo-1,zo-1,0,0,0);
LED(xo-1,yo+1,zo-1,0,0,0);
LED(xo-1,yo-1,zo-1,0,0,0);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x2o+1,y2o+1,z2o,0,0,0);
LED(x2o+1,y2o-1,z2o,0,0,0);
LED(x2o-1,y2o+1,z2o,0,0,0);
LED(x2o-1,y2o-1,z2o,0,0,0);
LED(x2o+1,y2o+1,z2o+1,0,0,0);
LED(x2o+1,y2o-1,z2o+1,0,0,0);
LED(x2o-1,y2o+1,z2o+1,0,0,0);
LED(x2o-1,y2o-1,z2o+1,0,0,0);
LED(x2o+1,y2o+1,z2o-1,0,0,0);
LED(x2o+1,y2o-1,z2o-1,0,0,0);
LED(x2o-1,y2o+1,z2o-1,0,0,0);
LED(x2o-1,y2o-1,z2o-1,0,0,0);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x,y,z,c1,c2,c3);
LED(x,y,z-1,c1,c2,c3);
LED(x,y,z+1,c1,c2,c3);
LED(x,y,z-2,c1,c2,c3);
LED(x,y,z+2,c1,c2,c3);
LED(x+1,y,z,c1,c2,c3);
LED(x-1,y,z,c1,c2,c3);
LED(x,y+1,z,c1,c2,c3);
LED(x,y-1,z,c1,c2,c3);
LED(x+2,y,z,c1,c2,c3);
LED(x-2,y,z,c1,c2,c3);
LED(x,y+2,z,c1,c2,c3);
LED(x,y-2,z,c1,c2,c3);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x+1,y+1,z,c1,c2,c3);
LED(x+1,y-1,z,c1,c2,c3);
LED(x-1,y+1,z,c1,c2,c3);
LED(x-1,y-1,z,c1,c2,c3);
LED(x+1,y+1,z+1,c1,c2,c3);
LED(x+1,y-1,z+1,c1,c2,c3);
LED(x-1,y+1,z+1,c1,c2,c3);
LED(x-1,y-1,z+1,c1,c2,c3);
LED(x+1,y+1,z-1,c1,c2,c3);
LED(x+1,y-1,z-1,c1,c2,c3);
LED(x-1,y+1,z-1,c1,c2,c3);
LED(x-1,y-1,z-1,c1,c2,c3);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x2,y2,z2,c21,c22,c23);
LED(x2,y2,z2-1,c21,c22,c23);
LED(x2,y2,z2+1,c21,c22,c23);
LED(x2,y2,z2-2,c21,c22,c23);
LED(x2,y2,z2+2,c21,c22,c23);
LED(x2+1,y2,z2,c21,c22,c23);
LED(x2-1,y2,z2,c21,c22,c23);
LED(x2,y2+1,z2,c21,c22,c23);
LED(x2,y2-1,z2,c21,c22,c23);
LED(x2+2,y2,z2,c21,c22,c23);
LED(x2-2,y2,z2,c21,c22,c23);
LED(x2,y2+2,z2,c21,c22,c23);
LED(x2,y2-2,z2,c21,c22,c23);
if (runMode != inMode ) {return;} // drop out if mode changes 
LED(x2+1,y2+1,z2,c21,c22,c23);
LED(x2+1,y2-1,z2,c21,c22,c23);
LED(x2-1,y2+1,z2,c21,c22,c23);
LED(x2-1,y2-1,z2,c21,c22,c23);
LED(x2+1,y2+1,z2+1,c21,c22,c23);
LED(x2+1,y2-1,z2+1,c21,c22,c23);
LED(x2-1,y2+1,z2+1,c21,c22,c23);
LED(x2-1,y2-1,z2+1,c21,c22,c23);
LED(x2+1,y2+1,z2-1,c21,c22,c23);
LED(x2+1,y2-1,z2-1,c21,c22,c23);
LED(x2-1,y2+1,z2-1,c21,c22,c23);
LED(x2-1,y2-1,z2-1,c21,c22,c23);
if (runMode != inMode ) {return;} // drop out if mode changes 
x2o=x2;
y2o=y2;
z2o=z2;

xo=x;
yo=y;
zo=z;

delay(45);

x = x+xmult;
y= y+ymult;
z= z+zmult;

x2 = x2+x2mult;
y2= y2+y2mult;
z2= z2+z2mult;

if(x>=7){
//x=7;
xmult=random(-1,1);
}
if(y>=7){
//y=7;
ymult=random(-1,1);}
if(z>=7){
//z=7;
zmult=random(-1,1);}
if(x<=0){
//x=0;
xmult=random(0,2);}
if(y<=0){
//y=0;
ymult=random(0,2);}
if(z<=0){
//z=0;
zmult=random(0,2);
}

if(x2>=7){
//x=7;
x2mult=random(-1,1);
}
if(y2>=7){
//y=7;
y2mult=random(-1,1);}
if(z2>=7){
//z=7;
z2mult=random(-1,1);}
if(x2<=0){
//x=0;
x2mult=random(0,2);}
if(y2<=0){
//y=0;
y2mult=random(0,2);}
if(z<=0){
//z=0;
z2mult=random(0,2);
}

}//counter  counter counter counter counter

for(counter=0; counter<15; counter++){
  color_select=random(0,3);
if(color_select==0){
c1 = 0;
c2 = random(0,16);
c3 = random(0,16);}
if(color_select==1){
c1 = random(0,16);
c2 = 0;
c3 = random(0,16);}
if(color_select==2){
c1 = random(0,16);
c2 = random(0,16);
c3 = 0;}
  
  
int num1=-1, num2=-4, num3=-6, num4=-10;
for(m=0; m<20; m++){
  
num1++;
num2++;
num3++;
num4++;


for(i=3; i<5; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,3,num1,0,0,0);
LED( 3,i,num1,0,0,0);
LED( 4,i,num1,0,0,0);
LED( i,4,num1,0,0,0);
}
for(i=3; i<5; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,4,num1+1,c1,c2,c3);
LED( 4,i,num1+1,c1,c2,c3);
LED( 3,i,num1+1,c1,c2,c3);
LED( i,3,num1+1,c1,c2,c3);
}
for(i=2; i<6; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,2,num2,0,0,0);
LED( 2,i,num2,0,0,0);
LED( 5,i,num2,0,0,0);
LED( i,5,num2,0,0,0);
}
for(i=2; i<6; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,2,num2+1,c1,c2,c3);
LED( 2,i,num2+1,c1,c2,c3);
LED( 5,i,num2+1,c1,c2,c3);
LED( i,5,num2+1,c1,c2,c3);
}
for(i=1; i<7; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,1,num3,0,0,0);
LED( 1,i,num3,0,0,0);
LED( 6,i,num3,0,0,0);
LED( i,6,num3,0,0,0);
}
for(i=1; i<7; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,1,num3+1,c1,c2,c3);
LED( 1,i,num3+1,c1,c2,c3);
LED( 6,i,num3+1,c1,c2,c3);
LED( i,6,num3+1,c1,c2,c3);
}
for(i=0; i<8; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,0,num4,0,0,0);
LED( 0,i,num4,0,0,0);
LED( 7,i,num4,0,0,0);
LED( i,7,num4,0,0,0);
}
for(i=0; i<8; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,0,num4+1,c1,c2,c3);
LED( 0,i,num4+1,c1,c2,c3);
LED( 7,i,num4+1,c1,c2,c3);
LED( i,7,num4+1,c1,c2,c3);
}
//delay(1);
}//m

num1=8;
num2=11;
num3=13;
num4=17;

for(m=0; m<20; m++){
num1--;
num2--;
num3--;
num4--;
for(i=3; i<5; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,3,num1,0,0,0);
LED( 3,i,num1,0,0,0);
LED( 4,i,num1,0,0,0);
LED( i,4,num1,0,0,0);
}
for(i=3; i<5; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,4,num1-1,0,0,15);
LED( 4,i,num1-1,0,0,15);
LED( 3,i,num1-1,0,0,15);
LED( i,3,num1-1,0,0,15);
}
for(i=2; i<6; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,2,num2,0,0,0);
LED( 2,i,num2,0,0,0);
LED( 5,i,num2,0,0,0);
LED( i,5,num2,0,0,0);
}
for(i=2; i<6; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,2,num2-1,0,0,15);
LED( 2,i,num2-1,0,0,15);
LED( 5,i,num2-1,0,0,15);
LED( i,5,num2-1,0,0,15);
}
for(i=1; i<7; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED(  i,1,num3, 0, 0, 0);
LED(  1,i,num3, 0, 0, 0);
LED(  6,i,num3, 0, 0, 0);
LED(  i,6,num3, 0, 0, 0);
}
for(i=1; i<7; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,1,num3-1, 0, 0, 15);
LED( 1,i,num3-1, 0, 0, 15);
LED( 6,i,num3-1, 0, 0, 15);
LED( i,6,num3-1, 0, 0, 15);
}
for(i=0; i<8; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,0,num4,0,0,0);
LED( 0,i,num4,0,0,0);
LED( 7,i,num4,0,0,0);
LED( i,7,num4,0,0,0);
}
for(i=0; i<8; i++){
if (runMode != inMode ) {return;} // drop out if mode changes   
LED( i,0,num4-1,0,0,15);
LED( 0,i,num4-1,0,0,15);
LED( 7,i,num4-1,0,0,15);
LED( i,7,num4-1,0,0,15);
}
//delay(1);
}//m
}//counter
}//harlem SHake

//***************************************************************************************************************************
// Music Functions
//***************************************************************************************************************************
void spectrum1 (int iterations, int delayx) {
  
  byte rarray [8] = { 0,  0,  2,  2,  0,  0, 15,  0};      // originally {0,0,8,8,0,0,63,16} divide by 4 to emulate chipkit values.
  byte garray [8] = {15,  0,  8,  0,  2,  8,  0,  8};      // originally {63,0,32,0,8,32,0,63)
  byte barray [8] = { 0,  4,  0,  8,  8,  0,  0  ,13};     // originally {0,63,0,32,32,0,0,63}

  int inMode=runMode;
  
  while(iterations) {

    readmsgeq7();
    for (byte freq=0;freq<8;freq++) 
         {
         if (runMode != inMode ) {return;} // drop out if mode changes
         wall (freq,VUtopb[freq],rarray[freq],garray[freq],barray[freq]); // number from 0 to just under 9 approx for height of wall with colour from RGB array
         }
    zdown(1);
    if (delayx > 0) {delay (delayx);} // if we aren't running full tilt, delay.
    //clean top layer
    cleartop();
    iterations--;
    }
 }


void spectrum2 (int iterations, int sdelay) {
  int inMode=runMode;
  while(iterations)
    {
    if (runMode != inMode ) {return;} // drop out if mode changes
  //clean top row
    cleartop();
    zdown(1);
    readmsgeq7();
    if (runMode != inMode ) {return;} // drop out if mode changes
    fourSquare (0,0,VUtop[2],15,0,0); // column 4X4 with height of frequency 2 with colour 32(R), 0(G), 0(B)...this is the red column
    fourSquare (4,4,VUtop[3],10,10,0); // column 4X4 with height of frequency 3 yellow
    fourSquare (0,4,VUtop[4],0,15,0); // column 4X4 with height of frequency 5 green
    fourSquare (4,0,VUtop[5],0,0,15); // column 4X4 with height of frequency 4 blue
    delay (sdelay);
    iterations--;
    }
}


void spectrum3 (int iterations, int delayx) 
  {  
  byte rarray [8] = { 0,  0,  2,  2,  0,  4, 15,  0};
  byte garray [8] = {15,  0,  8,  0, 15, 15,  0,  8};
  byte barray [8] = { 0, 15,  0,  8,  8, 15,  0, 13};

  int inMode=runMode;
  while (iterations) {
       if (runMode != inMode ) {return;} // drop out if mode changes
       readmsgeq7();
       for (int freq=0;freq<8;freq++) {
       for (int z=0; z<VUtopb[freq];z++) {// draw column to height of frequency
         if (runMode != inMode ) {return;} // drop out if mode changes
         LED (freq,7, z, rarray[freq],garray[freq],barray[freq]); 
         }
        }
       xback(1);
  if (delayx > 0) {delay (delayx);}
  // cleartop();
  iterations--;
  }
 }


void spectrum3a (int iterations, int delayx) 
  { // spectrum3a is similar to 3 except that it goes on a different axis and faster.
  byte rarray [8] = { 0, 0, 2, 2,  0,  4, 15, 0};
  byte garray [8] = {15, 0, 8, 0, 15, 15,  0, 8};
  byte barray [8] = { 0,15, 0, 8,  8, 15,  0, 13};

  int inMode=runMode;
  
  while (iterations) 
       {
       if (runMode != inMode ) {return;} // drop out if mode changes
       readmsgeq7();
       for (byte freq=0;freq<8;freq++) {
       for (byte z=0; z<VUtopb[freq];z++) {// draw column to height of frequency
         if (runMode != inMode ) {return;} // drop out if mode changes
         LED (0,freq, z, rarray[freq],garray[freq],barray[freq]); // panels 0 - 3 (first 4)
         }
        } 
       yback(1);
       if (delayx > 0) {delay (delayx);}
       for(byte lev=0; lev<8; lev++)
          for (byte r=0; r<8; r++)
              {
              LED (0, r, lev, 0, 0, 0); 
              }
       iterations--;
       }
 }


 void spectrum4 (int iterations, int sdelay) {
  int inMode=runMode;
  while (iterations) {
    if (runMode != inMode ) {return;} // drop out if mode changes
  //clean top row
    cleartop();
    zdown(1);
    readmsgeq7();
    if (runMode != inMode ) {return;} // drop out if mode changes
    fourSquare2 (0,0,VUtop[2],8,0,0); // column 4X4 with height of frequency 2 with colour 32(R), 0(G), 0(B)...this is the red column
    fourSquare2 (0,4,VUtop[5],0,15,0); // column 4X4 with height of frequency 5 green
    fourSquare2 (4,0,VUtop[4],0,0,15); // column 4X4 with height of frequency 4 blue
    fourSquare2 (4,4,VUtop[3],4,15,0); // column 4X4 with height of frequency 3 yellow
    fourSquare2 (2,2,VUtop[0],8,15,15); // column 4X4 with height of frequency 0 white
    delay (sdelay);
    // zdown(1);
    iterations--;
  }
}


void spectrum5 (int iterations, int delayx) {
  byte rarray [8] = {0,0,8,63,0,16,8};
  byte garray [8] = {63,0,32,0,64,63,0};
  byte barray [8] = {0,63,0,0,32,63,32};

  int inMode=runMode;
  while (iterations)
    {
    if (runMode != inMode ) {return;} // drop out if mode changes
    fill(0,0,0);
     readmsgeq7();
     for (byte freq=0;freq<7;freq++) {
       for (byte z=0; z<VUtop[freq];z++) {
        if (runMode != inMode ) {return;} // drop out if mode changes
        LED (freq, 7, z, rarray[freq],garray[freq],barray[freq]); // side 1
        LED (7-freq, 0, z, rarray[freq],garray[freq],barray[freq]);
        LED (0,freq,z,rarray[freq],garray[freq],barray[freq]);
        LED (7,7-freq,z,rarray[freq],garray[freq],barray[freq]); // side 4  
      }
     }
     delay (delayx);
     iterations--;
   }
 }


void spectrum5a (int iterations, int delayx) 
{ // spectrum5a is like 5 but the lower lights won't blink since we're not clearing at each step.
  byte rarray [8] = {0,0,8,63,0,16,8};
  byte garray [8] = {63,0,32,0,64,63,0};
  byte barray [8] = {0,63,0,0,32,63,32};

  int inMode=runMode;
  byte lastvutop[7] = {0, 0, 0, 0, 0, 0, 0};
  fill(0,0,0);
  while (iterations)
    {
    if (runMode != inMode ) {return;} // drop out if mode changes
     readmsgeq7();
     for (byte freq=0;freq<7;freq++) 
         {
         if(lastvutop[freq] < VUtop[freq])
           {
           for (byte z=lastvutop[freq]; z<VUtop[freq];z++) //consider starting at lastvutop[freq]
               {
               if (runMode != inMode ) {return;} // drop out if mode changes                
               LED (freq,7,z,rarray[freq],garray[freq],barray[freq]); // side 1
               LED (7-freq,0,z,rarray[freq],garray[freq],barray[freq]);
               LED (0,freq,z,rarray[freq],garray[freq],barray[freq]);
               LED (7,7-freq,z,rarray[freq],garray[freq],barray[freq]); // side 4  
               } // z
           } //if
        else
           {
           for(byte zz=VUtop[freq]; zz < 8; zz++)
              {
              if (runMode != inMode ) {return;} // drop out if mode changes
              LED (freq,7,zz,0,0,0); // side 1
              LED (7-freq,0,zz,0,0,0); // side 2
              LED (0,freq,zz,0,0,0); // side 3
              LED (7,7-freq,zz,0,0,0); // side 4
              } // zz
           }  //else
        lastvutop[freq] = VUtop[freq];
        } // for freq
     delay (delayx);
     iterations--;
   }
 }


 void spectrum6 (int iterations, int sdelay) {
  int inMode=runMode;
  while (iterations) {
    if (runMode != inMode ) {return;} // drop out if mode changes
    cleartop();
    zdown(1);
    readmsgeq7();
    twoSquare (6,0,VUtop[0],32,0,0); // column 2X2 with height of frequency 0 with colour 32(R), 0(G), 0(B)...this is the red column
    twoSquare (6,4,VUtop[1],0,63,0); 
    twoSquare (4,2,VUtop[2],0,0,63); 
    twoSquare (4,6,VUtop[3],32,63,0); 
    twoSquare (2,0,VUtop[4],0,63,63); 
    twoSquare (2,4,VUtop[5],32,0,63); 
    twoSquare (0,3,VUtop[6],32,63,63);
    delay (sdelay);
    iterations--;
  }
}


void spectrum7 (long iterations, int delayx, int rSpeed) 
{ // rSpeed = speed of rotation : higher = slower
  byte rarray [8] = {0,0,2,15,0,4,2};
  byte garray [8] = {15,0,8,0,15,15,0};
  byte barray [8] = {0,15,0,0,8,15,8};

  int inMode=runMode, rCount = 0;
  byte offset;
  while (iterations) {
    if (runMode != inMode ) {return;} // drop out if mode changes
    
    rCount ++;
    if (rCount > rSpeed) {rCount = 0;offset ++;} // If the counter reaches limit, reset it and increment the spectrum offset
    if (offset > 7) {offset = 0;}
     fill(0,0,0);
     readmsgeq7();
     for (byte freq=0;freq<7;freq++) 
         {
         for (byte z=0; z<VUtop[freq];z++) 
            {
            int yy = freq + offset;
            if (yy > 7) {yy=-7+freq+offset;}
            if (runMode != inMode ) {return;} // drop out if mode changes
            LED (yy,7,z,rarray[freq],garray[freq],barray[freq]);
            LED (7-yy,0,z,rarray[freq],garray[freq],barray[freq]);
            LED (0,yy,z,rarray[freq],garray[freq],barray[freq]);
            LED (7,7-yy,z,rarray[freq],garray[freq],barray[freq]);
            }
         }
 delay (delayx);  
 iterations--;
 }
 }

 
 void spectrum8 (int iterations, int sdelay) {
  float x, y, polar, base;
  int inMode=runMode;
  int whichbyte, wholebyte, whichbit;
  fill(0,0,0); // start with everything set to 0 since we're going to address red0[] etc directly.
  for (int times=0; times < iterations; times++) {
    if (runMode != inMode ) {break; } // drop out if mode changes
     //clean top row
    cleartop();
    zdown(1);
    readmsgeq7();
    base= (VUtop[0]+VUtop[1]) >> 1;  // bitshift 1 to divide by 2.
    if (base>4){base=4;}
    for (byte layer=0; layer<8; layer++){  // scan thru each layer
      for (byte column=0; column<8; column++){  // scan thru every column
        for (byte panel=0; panel<8; panel++)
          {  // scan thru every panel
          polar = sqrt((column-3.5)*(column-3.5)+(panel-3.5)*(panel-3.5)); // Calculate the distance squared

          if (polar>base){ // if an LED is outside the radius specified by base, turn it off. 
                 wholebyte=(layer << 6)+(panel << 3)+column;
                 whichbyte = int(wholebyte/8);
                 whichbit = wholebyte - (whichbyte << 3);

                 bitWrite(red3[whichbyte], whichbit, 0);
                 bitWrite(green3[whichbyte], whichbit, 0);
                 bitWrite(blue3[whichbyte], whichbit, 0);
                 // LED(layer, column, panel,0,0,0);
          }
          else{ // consider setting red0[3] only to cut the time by 25%.
           if (column<4 && panel<4) { 
                 wholebyte=( (VU[2]/virtualGain) << 6)+(panel << 3)+column;            
                 whichbyte = int(wholebyte/8);
                 bitWrite(red3[whichbyte], wholebyte-(whichbyte << 3), 1);
                  // LED(VU[2]/virtualGain, column, panel,15,0,0);
                 }
           if (column>3 && panel<4) { 
                 wholebyte=( (VU[3]/virtualGain) << 6)+(panel << 3)+column;            
                 whichbyte = int(wholebyte/8);
                 bitWrite(green3[whichbyte], wholebyte-(whichbyte << 3), 1);
                 // LED(VU[3]/virtualGain, column, panel,15,15,0);
              }
           if (column<4 && panel>3) { 
                 wholebyte=( (VU[4]/virtualGain) << 6)+(panel << 3)+column;            
                 whichbyte = int(wholebyte/8);
                 bitWrite(blue3[whichbyte], wholebyte-(whichbyte << 3), 1);
                // LED(VU[4]/virtualGain, column, panel,0,15,0);
              }
           if (column>3 && panel>3) { 
                 wholebyte=( (VU[5]/virtualGain) << 6)+(panel << 3)+column;            
                 whichbyte = int(wholebyte/8);
                 bitWrite(red3[whichbyte], wholebyte-(whichbyte << 3), 1);
                 // LED(VU[5]/virtualGain, column, panel,0,0,15);
              }
          }
        }
      }
    }
//    delay (sdelay); 
  } 
}



void spectrum9 (int iterations, int xdelay) { 
  int inMode=runMode;
  byte rarray [8] = { 0, 0, 2, 2,  0,  4, 15, 0};
  byte garray [8] = {15, 0, 8, 0, 15, 15,  0, 8};
  byte barray [8] = { 0,15, 0, 8,  8, 15,  0, 13};
  while(iterations)
   {
     if (runMode != inMode ) {return;} // drop out if mode changes
     fill(0,0,0);
     readmsgeq7();
     for (byte count=0; count<8; count++)
          {
          if (runMode != inMode ) {return;} // drop out if mode changes
          elasticPlane (VUtopb[count]/2, count, rarray[count], garray[count], barray[count]);
          }
     delay (xdelay); 
     iterations--;
   }
 }  


void spectrum10 (int iterations, int xdelay) { 
  int inMode=runMode;
  byte rarray [8] = { 0, 0, 2, 2,  0,  4, 15, 0};
  byte garray [8] = {15, 0, 8, 0, 15, 15,  0, 8};
  byte barray [8] = { 0,15, 0, 8,  8, 15,  0, 13};
   while(iterations)
   {
     if (runMode != inMode ) {return;} // drop out if mode changes
     fill(0,0,0);
     readmsgeq7();
     for (byte count=0; count<8; count++)
         {
         if (runMode != inMode ) {return;} // drop out if mode changes
         elasticPlane2 (VUtopb[count]/2, count, rarray[count], garray[count], barray[count]);
         }
     delay (xdelay); 
     iterations--;
   }
 }   


void beat2 (int iterations, int xdelay) {
 int inMode = runMode;
  while (iterations) 
    {
    if (runMode != inMode ) {return;} // drop out if mode changes
    fill(0,0,0);
    BeatPause();
    fill ( random(256)*random(2), random(256)*random(2),random(256)*random(2));
    delay (xdelay); // delay seems to only make it brighter when the delay is bigger.
    iterations--;
    }
}


// ****************************************************************************************************
// MUSIC SUPPORT ROUTINES
//*****************************************************************************************************

void BeatPause () 
{ // Other methods for pausing until a beat don't seem to work right, so I made this.
  readmsgeq7(); 
  while ((beat == 0) ) 
        { // if there's no beat and the mode hasn't changed, sit here.
        readmsgeq7();          
        }
}


void cleartop()
{
for (byte i=56; i<64; i++)
    {
    red0[i] = 0;
    red1[i] = 0;
    red2[i] = 0;
    red3[i] = 0;
    green0[i] = 0;
    green1[i] = 0;
    green2[i] = 0;
    green3[i] = 0;
    blue0[i] = 0;
    blue1[i] = 0;
    blue2[i] = 0;
    blue3[i] = 0;
    }
}


void elasticPlane (byte pwidth, byte pheight, byte R, byte G, byte B) 
{
  for (int xx = 4-pwidth; xx < (4+pwidth); xx++)
      {
      for (int yy = 4-pwidth; yy < (4+pwidth); yy++)
          {
          LED (yy,xx,pheight,R,G,B);
          }
      }
}


void elasticPlane2 (byte pwidth, byte pheight, byte R, byte G, byte B) {
  float x, y, polar, pwidthsquared;
  pwidthsquared = pwidth*pwidth;  // use this to eliminate the call to sqrt.
  for (unsigned char xx = 0; xx < 8; xx++) {
    for (unsigned char yy = 0; yy < 8; yy++){
      // originally, polar was sqrt of all that.  By eliminating the sqrt and simply
      // comparing against the pwidth squared, we can eliminate a time intensive function.
      polar = (xx-3.5)*(xx-3.5)+(yy-3.5)*(yy-3.5); // Calculate the distance (now squared)
      if (polar<pwidthsquared){ // if an LED is inside the radius specified by base, turn it on. 
        LED(yy, xx, pheight, R, G, B);
      }
    }
  }
}


void fill(byte red, byte green, byte blue)
{
  // this is used to fill the cube with random colors because we're not specifying the bits within the bytes.
  // for an easy clear, set arguments to 0.
for (byte i=0; i<64; i++)
  {
  red0[i] = red;
  red1[i] = red;
  red2[i] = red;
  red3[i] = red;
  green0[i] = green;
  green1[i] = green;
  green2[i] = green;
  green3[i] = green;
  blue0[i] = blue;
  blue1[i] = blue;
  blue2[i] = blue;
  blue3[i] = blue;
  }
}


void fourSquare (byte x, byte y, byte z, byte R, byte G, byte B){ //draw a pillar at x,y with height z, colour R,G,B
  if (x>4) {x=4;}
  if (y>4) {y=4;}
  if (z >8 ) z=8;
  if (R>15) {R=15;}
  if (G>15) {G=15;}
  if (B>15) {B=15;}
  
  if (z>0) {
  for (byte zz=0; zz<z;zz++){
    for (byte xx=0;xx<4;xx++) {
      for (byte yy=0;yy<4;yy++) {
            LED (x+xx, y+yy, zz, R,G,B);
        }
      }
    }
  } 
}


void fourSquare2 (byte x, byte y, byte z, byte R, byte G, byte B){ //draw a square at x,y at height z, colour R,G,B
  if (x>4) {x=4;}
  if (y>4) {y=4;}
  if (R>15) {R=15;}
  if (G>15) {G=15;}
  if (B>15) {B=15;} 
  for (byte xx=0;xx<4;xx++) {
    for (byte yy=0;yy<4;yy++) {
      LED (y+yy, x+xx, z-1, R,G,B);
     }
  }
}


void readmsgeq7()
{
int GAIN;
//digitalWrite (A1, LOW);  // chip has been reset - set MSGEQ7 chip reset to low
PORTF &= ~_BV(PF1);        // chip has been reset - set MSGEQ7 chip reset to low  This is faster than digitalWrite
delayMicroseconds(72);                             
for (byte mfreq = 0; mfreq < 7; mfreq++) 
    { // read 7 frequencies
//   digitalWrite (A3, LOW);    // toggle clock low to allow next freq. read
     PORTF &= ~_BV(PF3);        // toggle clock low to allow next freq. read  This is faster than digitalWrite
     delayMicroseconds(36);
     VU[mfreq] = analogRead(A0); // reset and clock must be low, so read value into array at current point
     VUtop[mfreq] = min(VU[mfreq]/virtualGain, 8); // make sure VUtop is not higher than 8 if gain if high and input is high
//    VUtop[mfreq] = VU[mfreq]/virtualGain;   // added -1 to allow for 0 since it goes from 1
//    if (VUtop[mfreq] > 8) {VUtop[mfreq] = 8;} // make sure VUtop is not higher than 8 if gain if high and input is high
//     digitalWrite (A3, HIGH); // toggle clock HIGH to prepare for read
     PORTF |= _BV(PF3);         // toggle clock HIGH to prepare for read  This is faster than digitalWrite
     delayMicroseconds(18);
     }

// Create alternate VUtop array to include a phantom frequency between 0 and 1 for those effects using 8 frequencies.
VUtopb[0] = VUtop[0];
VUtopb[1] = min((VU[0]+VU[1])/(virtualGain+virtualGain), 8);
for (byte mfreq=2; mfreq<8; mfreq++)
     VUtopb[mfreq] = VUtop[mfreq-1];

//digitalWrite (A1, HIGH); // set MSGEQ7 chip reset to high - 
PORTF |= _BV(PF1);         // set MSGEQ7 chip reset to high - This is faster than digitalWrite
                           // reset it because we have read all frequencies, 
                           // and hold it in reset until we need to read it again
GAIN = analogRead (A5);  // 
if (GAIN > 8) {virtualGain = 128 - (GAIN/8);} // when analog pin is not used, leave the virtualGain alone - 
                                              // otherwise make it a ## from 1 to 128 subtracted from the gain.
if (VU[1] < GAIN){beat = 0;} // beat detect routine
if (VU[1] > GAIN){beat = 1;} // beat detect routine
}


void twoSquare (byte x, byte y, byte z, byte R, byte G, byte B){ //draw a 2X2 pillar at x,y with height z, colour R,G,B
  if (x>6) {x=6;}
  if (y>6) {y=6;}
  if (R>15) {R=15;}
  if (G>15) {G=15;}
  if (B>15) {B=15;}
  
  if (z>0) {
  for (byte zz=0; zz<z;zz++){
    for (byte xx=0;xx<2;xx++) {
      for (byte yy=0;yy<2;yy++) {
        LED (y+yy, x+xx, zz, R,G,B);
        }
      }
    }
  } 
}


void wall (byte x,byte y, byte R, byte G, byte B) // make a "wall" at position X, Y high
{
  byte w;
  byte z;
  for (z=0;z < y;z++)
    {
    for (w=0;w < 8;w++)
      {
      LED (w,x,z,R,G,B); // level, row, column
      }
    }
}


void xback (int times)
//shift entire cube back (times) layer(s)
{
for (unsigned char i=0; i<64; i++)
    {
    for (unsigned char j=0; j<7; j++)
        {
         bitWrite (red0[i], j, bitRead(red0[i], j+1));
         bitWrite (red1[i], j, bitRead(red1[i], j+1));
         bitWrite (red2[i], j, bitRead(red2[i], j+1));
         bitWrite (red3[i], j, bitRead(red3[i], j+1));
         bitWrite (green0[i], j, bitRead(green0[i], j+1));
         bitWrite (green1[i], j, bitRead(green1[i], j+1));
         bitWrite (green2[i], j, bitRead(green2[i], j+1));
         bitWrite (green3[i], j, bitRead(green3[i], j+1));
         bitWrite (blue0[i], j, bitRead(blue0[i], j+1));
         bitWrite (blue1[i], j, bitRead(blue1[i], j+1));
         bitWrite (blue2[i], j, bitRead(blue2[i], j+1));
         bitWrite (blue3[i], j, bitRead(blue3[i], j+1));
        }
    // need to clear panel 7 after everything is moved.
    bitWrite (red0[i], 7, 0);
    bitWrite (red1[i], 7, 0);
    bitWrite (red2[i], 7, 0);
    bitWrite (red3[i], 7, 0);
    bitWrite (green0[i], 7, 0);
    bitWrite (green1[i], 7, 0);
    bitWrite (green2[i], 7, 0);
    bitWrite (green3[i], 7, 0);
    bitWrite (blue0[i], 7, 0);
    bitWrite (blue1[i], 7, 0);
    bitWrite (blue2[i], 7, 0);
    bitWrite (blue3[i], 7, 0);
    }
}


void yback (unsigned char times)
{
  for (byte i=7; i>0; i--)
      {
      for (byte j=0; j<8; j++)
          {
          red0[i+j*8] = red0[i+j*8-1];
          red1[i+j*8] = red1[i+j*8-1];
          red2[i+j*8] = red2[i+j*8-1];
          red3[i+j*8] = red3[i+j*8-1];
          green0[i+j*8] = green0[i+j*8-1];
          green1[i+j*8] = green1[i+j*8-1];
          green2[i+j*8] = green2[i+j*8-1];
          green3[i+j*8] = green3[i+j*8-1];
          blue0[i+j*8] = blue0[i+j*8-1];
          blue1[i+j*8] = blue1[i+j*8-1];
          blue2[i+j*8] = blue2[i+j*8-1];
          blue3[i+j*8] = blue3[i+j*8-1];
          }
      }
}


void zdown (int times) 
{ //shift entire cube down (times) layer(s).  
while (times)
    {
    for (unsigned char i=0; i<56; i++)
        {
        red0[i]   = red0[i+8];
        red1[i]   = red1[i+8];
        red2[i]   = red2[i+8];
        red3[i]   = red3[i+8];
        green0[i] = green0[i+8];
        green1[i] = green1[i+8];
        green2[i] = green2[i+8];
        green3[i] = green3[i+8];
        blue0[i]  = blue0[i+8];
        blue1[i]  = blue1[i+8];
        blue2[i]  = blue2[i+8];
        blue3[i]  = blue3[i+8];
        }
    times--;
    }
}



